/* autodet.c - detekce kodovani (program cz2cz) */

/*
 * cz2cz tools
 * Copyright (C) 2005 Vlastimil Krejcir
 *
 * This file is part of cz2cz.
 *
 * cz2cz is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include<stdio.h>
#include<gdbm.h>
#include<string.h>
#include<ctype.h>
#include<stdlib.h>

#include"cz2czdef.h"

/* pocet ASCII kodovani */
#define NO_OF_ASCII_CODS 4 
/* pocet vsech kodovani */
#define TOTAL_CODS 5

/* potrebna makra pro detekci kodovani UTF8 */
#define UTF8_BYTE1_MASK 224
#define UTF8_BYTE2_MASK 192
#define UTF8_BYTE1_RES 192
#define UTF8_BYTE2_RES 128
#define UTF8_INDEX 4

#define LADENI 1

int is_in_alphabet(unsigned char ch, int ind) 
{ int win[5]={138, 141, 142, 154, 157};
  int iso[5]={171, 174, 185, 187, 190};
  int lat[13]={156, 167, 172, 183, 212, 213, 214, 222, 224, 229, 230, 231, 252};
  int kam[16]={128, 131, 134, 135, 136, 137, 139, 143, 145, 146, 149, 150, 152, 164, 165, 168};
  int j;
  
  switch (ind) {
  case 0: for (j=0; j!=7; j++) if (ch == iso[j]) return 1;
          return 0;
  case 1: for (j=0; j!=6; j++) if (ch == win[j]) return 1;
          return 0; 
  case 2: for (j=0; j!=13; j++) if (ch == lat[j]) return 1;
          return 0;
  case 3: for (j=0; j!=15; j++) if (ch == kam[j]) return 1;
          return 0;
  default: return -1;
  }
  return 0;
}
 
int candidate_simple(unsigned char z) /* fce na testovani, jestli je pismeno mozny kandidat na testovani databazi */
{ int tab[24]={185, 187, 190, 169, 171, 174,
               154, 157, 158, 138, 141, 142,
               231, 156, 167, 230, 155, 166,
               168, 159, 145, 155, 134, 146};
  int j;

  if (z < 128) {
    if (isalpha(z)) return 0; /* vporadku je to pismeno */
    else return -1; /* neni pismeno */
  }
  else {
    for (j=0; j!=24; j++) { if (z == tab[j]) return 1; }
    return 0;
  }
}

int autodetect_line_ends(FILE *fil) /* detekce koncu radku v souboru */
{
	char c, prev_c;
	int cr=13;
	int lf=10;
	
	prev_c=getc(fil);
	while ((c=getc(fil)) != EOF) {
		if (c == lf) {
			if (prev_c == cr) return CRLF;
			else return LF;
		}
		else if (prev_c == cr) return CR; /* jsou-li konce radku jen CR */
		prev_c=c;
	}
	
	return CZ_CANTDETTECTLINEENDS;
}

int autodetect(FILE *fil) /* vraci CZ_DBFATALERROR pri neotevreni databaze, 0 pri nepovedene detekci 
                             kladne hodnoty pri detekci databazi a zaporne hodnoty pri detekci simple */
{ 
  /* promenne datab. algoritmu */
  GDBM_FILE database;
  datum key, content;
  unsigned char str[STR_LENGHT], p[STR_LENGHT]; /* retezce na upravu slov */
  int i, j, beg, end, pom=0; /* indexove promenne */
  int dbdetect_flag=0; /* ma smysl testovat slovo v databazi ? vyskytl se ve slove potrebny znak ? */
  
  /* promenne jednoducheho algoritmu */
  int top_index=0, k; /* nejvyssi max[top_index] */  
  unsigned long max[TOTAL_CODS], counter=0; /* pole pro ukladani retezcu (znaku kodovani) a pomocna promenna */
  int ascii_flag=1; /* je kodovani v ascii */
  
  for (i=0; i!=TOTAL_CODS; i++) max[i]=0; /* vynuluj pole citacu */
  
  if ((database=gdbm_open(CZ_DATAB, 512, GDBM_READER, 0644, NULL)) == NULL) {
    return CZ_DBFATALERROR;    
  }
  
  while (fscanf(fil, "%1023s", str) != EOF) { /* nacitej slova ze souboru, slova delky nejvyse 1023 znaku */
   
    /* plug-in jednoducheho algoritmu */
      /* ------------------------------ */
    for (i=0; i<strlen(str); i++) {
    /* ---test na UTF-8 -- */
    /* ---muzeme v testu pouzit nasledujici znak -- v nejhorsim na konci retezce neprelezeme do haje, pac je tam '\0' */
    if (((str[i] & UTF8_BYTE1_MASK) == UTF8_BYTE1_RES) && ((str[i+1] & UTF8_BYTE2_MASK) == UTF8_BYTE2_RES)) max[UTF8_INDEX]++;
     
    /* --- testujeme ASCII kodovani --- */
    if (str[i] > 127) { /* pokud jsou z horni casti tabulky */
        ascii_flag=0; /* vynuluj flag pro urceni 7-bit ascii */
        for (k=0; k!=NO_OF_ASCII_CODS; k++) /* projdi pres vsechny mnoziny kodovani */
          if (is_in_alphabet(str[i], k) == 1) max[k]++; /* testuj je-li znak v mnozine, zvys citac */
      }
    }
    /* ------------------------------ */

    beg=0; end=strlen(str); /* nastav konec a zacatek */
    
    if (ispunct(str[beg])) beg=1;/* je-li prvni znak interpunkci znamenko */
    if (ispunct(str[end-1])) end=end-1;  /* je-li posledni znak int. znam. */
    
    for (i=beg, j=0; i!=end; i++, j++) { /* projdi vsechny znaky */
      pom=candidate_simple(str[i]);
      p[j]=str[i]; /* postupne si to zkopiruji do promenne p */
      if (pom == 1) dbdetect_flag=1; /* je-li tam kandidatsky znak */
      else if (pom < 0) { dbdetect_flag=0; break; } /* je-li tam neco jineho nez je povoleno */ 
    } /*konec for */
    p[j]='\0'; /* zakonci kopir. retezec */

    /* pokracuj hledanim v databazi */
    if (dbdetect_flag == 1) {
      key.dptr=p;
      key.dsize=strlen(key.dptr)+1;
      content=gdbm_fetch(database, key);
	  
      if (content.dptr != NULL) {
				if (strcmp(content.dptr, "iso") == 0){ free((void *) content.dptr); gdbm_close(database); return CZ_ISO; }
				if (strcmp(content.dptr, "win") == 0){ free((void *) content.dptr); gdbm_close(database); return CZ_WIN; }
				if (strcmp(content.dptr, "lat") == 0){ free((void *) content.dptr); gdbm_close(database); return CZ_LAT; }
				if (strcmp(content.dptr, "kam") == 0){ free((void *) content.dptr); gdbm_close(database); return CZ_KAM; }
				/*
				if (strcmp(content.dptr, "ut8") == 0){ free((void *) content.dptr); gdbm_close(database); return CZ_UTF8; }
				*/
				free((void *) content.dptr);
			}
      dbdetect_flag=0;
    }
  }
  /* pokud skonci while() pak databazovy algoritmus neuspel, zkusime hadat jednoduse */
  gdbm_close(database); /* zavri dtatabzi */
  
  if (ascii_flag) return CZ_ASCII; /* pokud je 7bit ascii */
  
  /* najdi nejvetsi citac a jeho index */
  if ((max[0] == max[1]) && (max[1] == max[2]) && (max[2] == max[3]) && (max[3] == max[4])) return 0; /* nezdari-li se detekce */
  for (i=0; i!=TOTAL_CODS; i++) {
    if (max[i] > counter) { 
      counter=max[i];
    
      top_index=i;  
    } 
/*    printf("%d[%d]\n", max[i], i);*/
  }
  /* index odpovida hodnotam maker */
	/* najdeme-li UTF-8 pak je vse OK i bez DB */
  return top_index == UTF8_INDEX ? top_index+1 : (top_index+1)*(-1); 
}












