/* cz2cz - program pro prevod kodovani cestiny s autodetekci */
/*
 * cz2cz tools
 * Copyright (C) 2005 Vlastimil Krejcir
 *
 * This file is part of cz2cz.
 *
 * cz2cz is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<stdio.h>
#include<string.h>
#include<stdlib.h>
#if UNIX
 #include<sys/stat.h>
 #include<sys/types.h>
#endif
#if DOS
 #include<sys\stat.h>
 #include<sys\types.h>
#endif
#include<unistd.h>
#if LINUX
 #include<getopt.h>
#endif

#include"cz2czdef.h" /* definece maker pro cely program */
#include"autodet.h" /* autodet.h - fce pro autodetekci kodovani */
#include"errors.h" /*  errors.h chybova i jina hlaseni */
#include"conf.h" /* conf.h zpracovani konfiguracniho souboru */
#include"tables.h" /* prevody mezi kodovanimi */
#include"databadd.h" /* pridavani do databaze */
#include"version.h" /* verze */
#include"codes.h" /* pracuje s popisky kodovani */

#if UNIX
 #define CONFIG_HOME "/.cz2czrc" /* jmeno konfig. souboru */
 #define ADR_CONFIG getenv("HOME") /* promenna prostredi s adresarem konfig. souboru */
 #define CONFIG_SYS "/cz2czrc" /* jmeno konf. souboru v systemovem adfresari */
#endif
#if DOS
 #define CONFIG_HOME "cz2czrc.cnf" /* jmeno konfig. souboru */
 #define ADR_CONFIG "" /* pro DOS systemy */
#endif

/*#define MAX_P_NUM (1<<(sizeof(unsigned long)*8))*//* maximalni cislo zprac soub. pro volbu -m 2 a -p */
#define MAX_P_NUM (~ (unsigned long) 0)
#define CLEAN free((void *) pp); free((void *) out_fi); /*uvolneni pameti */
#define MAX 12 /* delka promenne pro retezec na spracoani cisla pro mod 3 */
#define TMP "cz2cztmp" /* temp soubor pro operace */
#define CZ_STDIN 1

/* globalni definice */
  int opt_f=0; /* flag pro prepisovani existujicich souboru - defaultne se tazeme */
  int line_ends=SAME; /* defaultne nastaveno na ponechani puvodniho */
  struct stat s; /* struktura pro hodnoty fce stat() */
  char *tmp_nam;
  int opt_q=0;/* flag pro volbu -q, tedy quiet, aneb nevypisuje se nic, defaultne vypnuto */

/*
void print_short_help(void)
{ printf("cz2cz (verze %s)\n", VER);
  printf("------------------\n");
  printf("cz2cz [-fhKMv] [-l <hod>] [-m <cislo> [-p <hod>]] [-z <zdr_kod>] [-k <cil_kod>]\n"
         "      [-o <cil>] [[-a] <zdroj(e)>]\n");
  printf("-H vytiskne podrobnou napovedu\n");
}
*/

int is_conf(const char *str) /* vraci 0 nen-li str platny soubor */
{
  if (((stat(str, &s) != 0)) || /* konfiguracni soubor neni v HOME */
      ( !(S_ISREG(s.st_mode)) )) return 0; /* nebo to neni reg. soubor */
  
  return 1;
}

void print_long_help(void)
{ 
  char *ps;
  
  if ((ps=(char *) malloc((strlen(ADR_CONFIG))+(strlen(CONFIG_HOME))+1)) == NULL) {
    fprintf(stderr ,"%s", geterr(CZ_NOTENOUGHMEM));
    exit(0);
  }
  strcpy(ps, ADR_CONFIG);
  strcat(ps, CONFIG_HOME);

  printf("cz2cz2                                                   Vlastimil Krejcir 2004\n");
  printf("-------------------------------------------------------------------------------\n");
  printf("Program pro prevod textovych souboru mezi kodovanimi cestiny,\n" 
         "prevod na sprezky a do TeXovske konvence.\n");
  printf("Syntaxe:\n");
  printf("cz2cz [-cCDfhMSuUvV] [-l <hod>] [-m <cislo> [-p <hod>]] [-s <zdr_kod>]\n"
         "      [-d <cil_kod>] [-o <cil>] [[-a] <zdroj(e)>]\n");
  printf("-a           pouze detekuj kodovani (ostatni volby jsou ignorovany,\n"
         "             nezalezi na nastaveni modu)\n");
  printf("-u           doplneni databaze pro automatickou detekci (info: \"cz2cz -U\")\n");
  printf("-c           vypise informace o formatu konfiguracniho souboru\n");
  printf("-f           bude bez dotazu prepisovat cilove soubory, pokud jiz existuji\n");
  printf("-h           zobrazi tuto napovedu\n");
  printf("-C           vypise podporovana kodovani\n");
  printf("-M           vypise mozne mody (volba -m) pro prevod souboru\n");
  printf("-D           vypise pocet slov v databazi, podle nichz se provadi autodetekce\n");
#if UNIX  
  if (! is_conf(ps))
    printf("-S           vypise aktualni nastaveni (podle " SYSCONFDIR CONFIG_SYS ")\n");
  else printf("-S           vypise aktualni nastaveni (podle %s)\n", ps);
#endif
#if DOS
  printf("-S           vypise aktualni nastaveni (podle " CONFIG_HOME ")\n");
#endif
  printf("-V           vypise info o programu (autor, verze, ...)\n");
  printf("-v           bude vypisovat informace o provadenych cinnostech\n");
  printf("-q           tichy mod, nevypisuje zadne informace (ani chybove hlasky)\n"); 
  printf("-l <hod>     urceni konce radku pro vystup; lf - <LF>, crlf - <CR><LF>\n");
  printf("-m <cislo>   cislo modu, ktere udava jakym zpusobem bude reseno pojmenovani\n"
         "             ciloveho souboru, vypis cisel modu s popisem pres volbu -M\n"); 
  printf("-p <hod>     pripona pro cilove soubory pri modu=2, startovni cislo pri modu=3\n");
  printf("-s <zdr_kod> explicitni vyber kodovani zdrojoveho(vych) souboru(u), vypina\n"
         "             autodetekci\n");
  printf("-d <cil_kod> kodovani, do ktereho bude vstup prevaden (defaultne ISO-8859-2)\n");
  printf("-o <cil>     cilovy soubor (je-li mod=0) nebo adresar (je-li mod={1, 2, 3})\n");    

  free((void *) ps);
}

void print_codes(void)
{ printf("Podporovana kodovani (hodnota - vyznam):\n");
  printf("----------------------------------------\n");
  printf("iso - ISO-8859-2\n");
  printf("win - Win-1250\n"); 
  printf("pc2 - PCLatin2 (cp852)\n");
  printf("kam - bratri Kamenickych\n");
  printf("utf8 - UTF8 Unicode (jen autodetekce)\n");
  printf("ascii - 7-bit ASCII (bez hacku a carek)\n");
  printf("post - (pouze vystupni) postfixove sprezky (nepodleha autodetekci)\n");
  printf("pref - (pouze vystupni )prefixove sprezky (nepodleha autodetekci)\n");
  printf("tex - (pouze vystupni) TeX-ovska konvence (nepodleha autodetekci)\n"); 
}

void print_modes(void)
{ 
  printf("cz2cz - mody pro pojmenovani ciloveho souboru\n");
  printf("----------------------------------------------------\n");
  printf("0 - defaultni mod, cilovy soubor je zadan pres volbu -o,\n"
         "    neni-li vlozen cilovy soubor pak jde vystup na stdout\n"
	 "    vstupni soubor je pouze jeden nebo zadny (pak bude cten stdin)\n");
  printf("1 - cilove soubory maji stejne jmeno jako zdrojove soubory\n"
         "    zdrojove soubory jsou prepsany cilovymi, pokud neni zadan adresar\n"
         "    pres volbu -o, jinak probehne zapis do zadaneho adresare\n");
  printf("2 - cilove soubory maji stejne jmeno jako zdrojove + priponu definovanou\n"
         "    pres volbu -p; defaultni pripona je 'cz'; lze zadat cilovy adresar pres\n" 
         "    volbu -o\n");
  printf("3 - cilove soubory maji stejne jmeno jako zdrojove + cislo zpracovavaneho\n"
         "    souboru; startovni cislo lze nastavit pres volbu -p; defaultne 0;\n"
         "    lze zadat cilovy adresar pres volbu -o; maximalni hodnota je %lu\n", MAX_P_NUM);
} 

void print_format(void)
{ printf("cz2cz - format konfiguracniho souboru\n");
  printf("-------------------------------------\n");
  printf("Program cz2cz podporuje konfiguracni soubor, ve kterem si muzete navolit casto\n"
#if UNIX
         "pouzivane hodnoty. Program se pokusi pri spusteni cist soubor \"" CONFIG_HOME "\"\n"
	 "v domovskem adresari.\nNenalezne-li soubor, pak pouzije soubor \"" CONFIG_SYS "\""
	 "v adresari\n%s.\n", SYSCONFDIR);
  printf("Pokud neni nalezen ani tento soubor, pouziji se defaultni hodnoty.\n");	 
#endif
#if DOS
         "pouzivane hodnoty. Program se pokusi pri spusteni cist soubor \"cz2czrc.cnf\"\n"
	 "v adresari programu. Nenalezne-li soubor, pak pouzije defaultni hodnoty nasta-\n");
  printf("vene natvrdo v programu jako defaultni.\n");
#endif  
  putchar('\n');
  printf("Format souboru \""CONFIG_HOME"\" (zaznam): ");
  printf("promenna_x = hodnota_x\n");
  printf("Zaznamy jsou prave v tomto formatu a oddelovacem je kazdy bily znak (pro pre-\n"
         "hlednost je doporuceno psat kazdy zaznam na samostatny radek).\n");
  printf("Radek zacinajici znakem \"#\" je ignorovan az do konce.\n");
  putchar('\n');
  printf("Povolene promenne a jejich hodnoty: \n");
  printf("zdr_kod   nastaveni zdrojoveho kodovani, nastaveni teto promenne vypina\n"
         "          automatickou detekci kodovani vstupniho souboru;\n"
	 "          vypis povolenych hodnot: \"cz2cz -C\"\n"
         "          (nelze vsak nastavit 'tex', 'post' a 'pref')\n");
  printf("cil_kod   nastaveni ciloveho kodovani;\n"
         "          vypis povolenych hodnot: \"cz2cz -C\"\n");
  printf("mod       nastaveni kopirovaciho modu;\n"
         "          vypis povolenych hodnot: \"cz2cz -M\"\n");
  printf("pripona   je-li mod nastaven na hodnotu 2, pak urcuje priponu cilovych souboru\n");
  printf("start     je-li mod nastaven na hodnotu 3, pak urcuje zacatek cislovani cilo-\n"
         "          vych souboru\n");
  printf("k_radku   hodnota \"crlf\" nastavi pro vystup konce radku na <CR><LF>,\n"
         "          hodnota \"lf\" nastavi konce radku na <LF>\n"
	 "          je-li radek zapoznamkovan, pouziji se konce radku zdrojoveho souboru\n"); 
}

void print_dbm_help(void) 
{ printf("cz2cz - doplneni databaze\n");
  printf("-------------------------\n");
  printf("Program dovoluje rozsirit databazi slov, ktera se pouziva pro automaticke\n"
         "detekovani kodovani cestiny. Tato akce muze zvysit uspesnost detekovani.\n");
  printf("Syntaxe:\n");
  printf("cz2cz [-v] -s <zdr_kod> -u <soubor1> [<soubor2> [ ... ]]\n");
  printf("Vyznam:\n");
  printf("Program projde textovy(e) soubor(y) <soubor1>, <soubor2>, ... . Pokusi se\n"
         "v nich nalezt vhodna slova, ktera by byla pouzitelna pro autodetekci a ty\n"
	 "zaradi do databaze. Soubor(y) <soubor1>, <soubor2>, ... jsou v kodovani\n" 
	 "<zdr_kod>, ktere musi byt takto explicitne zadano, kvuli pozdejsi spravnosti\n"
	 "detekce.\n");
  printf("Zadate-li i volbu -v, bude se za kazde slovo, ktere se prida do databaze,\n" 
         "vypisovat na standardni vystup \"#\" (hash).\n");
  printf("VAROVANI: nespravne zadane zdrojove kodovani souboru <soubor1>, <soubor2>, ...\n"
         "          muze vest k znehodnoceni autodetekce.\n");
  printf("POZNAMKA: Databazi lze doplnovat take pomoci programu cz2czget a cz2czput.\n"
         "          Tento zpusob je vhodny zejmena pokud si prejete pridat do vasi\n"
         "          databaze slova z jine (v praxi vetsi) databaze. Spustenim techto\n"
         "          dvou programu bez parametru ziskate blizsi napovedu.\n");
}     

/*
char *ret_charcod(int code)
{ switch (code) {
  case CZ_ISO: return "ISO-8859-2";
  case CZ_WIN: return "Win-1250";
  case CZ_LAT: return "PCLatin2";
  case CZ_KAM: return "Kamenickych";
  case CZ_UTF8: return "UTF8";
  case CZ_ASCII: return "7-bit ASCII";
  case CZ_POST: return "postfixove sprezky";
  case CZ_PREF: return "prefixove sprezky";
  case CZ_TEX: return "TeX konvence";
  default: return "nedef.";
  }
}

int ret_cod(char *code) 
{ if (strcmp(code, "win") == 0) return CZ_WIN;
  else if (strcmp(code, "utf8") == 0) return CZ_UTF8;
  else if (strcmp(code, "pc2") == 0) return CZ_LAT;
  else if (strcmp(code, "kam") == 0) return CZ_KAM;
  else if (strcmp(code, "iso") == 0) return CZ_ISO;
  else if (strcmp(code, "ascii") == 0) return CZ_ASCII;
  else if (strcmp(code, "post") == 0) return CZ_POST;
  else if (strcmp(code, "pref") == 0) return CZ_PREF;
  else if (strcmp(code, "tex") == 0) return CZ_TEX;
  else return -1;
}

int ret_end(char *line_end)
{ if (strcmp(line_end, "crlf") == 0) return CRLF;
  else if (strcmp(line_end, "lf") == 0) return LF;
  
  return -1;
}

char *ret_charend(int r)
{ if (r == CRLF) return "<CR><LF>";
  else if (r == LF) return "<LF>";
       else if (r == CR) return "<CR>";
            else if (r == CZ_CANTDETTECTLINEENDS) return "nezname konce radku";
	         else return "ponechat zdrojove";
}
*/

void print_settings(int m, int s, int d, unsigned long st, char *pr, int r, int z)
{ printf("mod = %d\n", m);
  if (z == 0)
    printf("zdrojove kodovani = %s\n", ret_charcod(s));
  else
    printf("zdrojove kodovani = autodetekce\n");
  printf("cilove kodovani = %s\n", ret_charcod(d));
  printf("konce radku = %s\n", ret_charend(r));
  printf("pripona (mod 2) = %s\n", pr);
  printf("start (mod 3) = %lu\n", st);
}


/* ---------------------------------- */
/* nasleduji fce pro praci se soubory */

int ret_dir(char *direct)
{ 
  if (stat(direct, &s) == 0)
    if (S_ISDIR(s.st_mode)) return 0; /* adr. existuje */
    else return 1; /* neni adresar */
  else return -1; /* neexistuje */
}

FILE *open_file(char *fil, char *fla, int *p_prep)
{ FILE *pom;
  char vol;
    
  *p_prep=0;
  /* test, zda jiz dany soubor neexistuje a mam-li se pri prepisu ptat */

    if (stat(fil, &s) == 0) { /* jestlize soubor existuje  */     
      if ((strcmp(fla, "wb") == 0) && (opt_f == 0)) {    
        printf("Soubor \"%s\" jiz existuje. Chcete jej prepsat (a/n)? ", fil);
	vol=getchar();
	while (getchar() != '\n'); /* docti buffer do konce a vse zahod */
	if ((vol == 'a') || (vol == 'y')) { /* soubor bude prepsan */
	  if ((pom=fopen(fil, fla)) == NULL) fprintf(stderr, "%s\"%s\"", geterr(CZ_CANTOPENF), fil);
	  return pom;
	}  
      	else { *p_prep=1; return NULL; } /* neprepisuj soubor */
      }     
    }  
    
    if ((pom=fopen(fil, fla)) == NULL) fprintf(stderr, "%s\"%s\"", geterr(CZ_CANTOPENF), fil);
    return pom;
}

char *ret_path(char *direct, char *file_n)
{ char *pom;
  
  /* free((void *) pom); pom=NULL; */
  if ((pom=(char *) malloc(strlen(direct)+strlen(file_n)+4)) == NULL) {
    fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM));
    return NULL;
  }  
  /* nasledujici fce pripravi jmeno souboru pro zapis do jineho adresare */
 
#if UNIX 
  strcpy(pom, "");
  strcat(pom, direct);
  strcat(pom, "/");
  strcat(pom, file_n);
#endif
#if DOS
  strcpy(pom, "");
  strcat(pom, direct);
  strcat(pom, "\\");
  strcat(pom, file_n);
#endif

  return pom;
} 

int readstdin2tmp(void) /* cti stdin do temp souboru */
{ FILE *tmp;
  char c;
  
  tmp_nam=tmpnam(NULL);
  if ((tmp=fopen(tmp_nam, "w")) == NULL) { 
    fprintf(stderr, "%s temp pro zapis.\n", geterr(CZ_CANTOPENF));
    return -1;
  }
  while ((c=getc(stdin)) != EOF) putc(c, tmp);
  
  if (fclose(tmp) == EOF) { fprintf(stderr, "%s temp.\n", geterr(CZ_CANTCLOSEF)); return -1; }
  
  return 0;
}  


/* -----------------------------------*/

int autod(const char *f, int flag) /* autodetekce pro prevody */
{ FILE *ff;  
  int h; 
  unsigned char c;
  
  if ((ff=fopen(f, "rb")) == NULL) { 
    fprintf(stderr, "cz2cz: Chyba: Nepodarilo se otevrit soubor \"%s\" pro cteni.\n", f);
    return -1;
  }
  
  line_ends=LF;
  h=0;	/* je to crlf nebo jen lf */
  while (fread(&c, sizeof(c), 1, ff)) {
    if ((h == 13) && (c == 10)) {
      line_ends=CRLF;
      break;    
    }
    h=c;
  }
       
  if (fclose(ff) == EOF) { fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTCLOSEF), f); return -1; }
  
  if ((ff=fopen(f, "r")) == NULL) { 
    fprintf(stderr, "cz2cz: Chyba: Nepodarilo se otevrit soubor \"%s\" pro cteni.\n", f);
    return -1;
  }					     
  else {
    h=autodetect(ff);
    if (fclose(ff) == EOF) { fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTCLOSEF), f); return -1; }
    if (h == CZ_DBFATALERROR) { return -1; } /* chyba databaze */
    if (h == 0) { /* detekce neuspesna */ 
      if (flag == CZ_STDIN) { fprintf(stderr, "%s", geterr(CZ_DETECTFAILED)); }
      else puterr(3, f); 
      return -2; 
    } 
    if (h < 0) {  /* detekce uspesna jednoduchym alg. */ 
      if (! opt_q) {
        if (flag == CZ_STDIN ) fprintf(stderr, "%s", getwar(CZ_SIMPLEDETECT));
        else puterr(2, f);
      }
      return h*(-1);	     
    }
    else {
      if (h > 0) return h; } /* detekce uspesna databazovym algoritmem */        
    
  }    
  return -1;
}  

int main(int argc, char **argv)
{ 
  extern char *optarg; /* promenne pro getopt */
  extern int optind; /* promenne pro getopt */
  
  /* promenne ziskane z konfiguracniho souboru */
  extern unsigned long c_start;
  extern int c_mode;
  extern char *c_source_code, *c_destination_code, *c_pp, *c_l_ends;
  
  int hlp; /* pomocna promenna */
  int prep=0; /* pomocna promenna pro zjistovani jestli uzivatel zvolil prepsat soubor 1, ci nikoli 0 */
  int i; /* pomocna indexova promenna pro for-cykly */
  char *pom_str; /* pomocny retezec */
  char c; /* pro getopt (volby) */
  FILE *fin, *fout; /* vstup a vystup */
  
  int mode=0; /* nastaveni modu pro jmena vystupnich souboru */
  char *out_fi;  /* pri volbe -o vystupni soubor */ 
  char *pp;      /* pripona vystupnich souboru pri mode=1 a volbe -p */
  int destination_code=1; /* cilove kodovani, defaultne ISO-8859-2 */ 
  int source_code=1; /* zdrojove kodovani */ 
  unsigned long p_num=0; /* startovni cislo pro vystupni soubor pri mode=2 a volbe -p */
  long db_words; /* slova pridava do databaze pri jejim rozsirovani */
  int l_ends=SAME; /* nastaveni l volby - konce radku, defaultne ponechat puvodni */
  int opt_a=0;/* flag pro volbu -a */
  int opt_v=0;/* flag pro volbu -v */
  int opt_u=0;/* flag pro volbu -d */
  int opt_s=1;/* flag pro  autodetekci */
  
  /* defaultni naplneni pripony */
  if ((pp=(char *) malloc(3)) == NULL) {
    fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM));
    return CZ_NOTENOUGHMEM;
  }
  strcpy(pp, "cz");
  out_fi=NULL; /* nastaveni ukazatele na jmeno vystupniho souboru */
  pom_str=NULL; /* nastaveni ukazatele pomocne promenne */

  /* ------ Kontrola databaze ------- */ 
  if (stat(CZ_DATAB, &s) != 0) {
    if (! opt_q) fprintf(stderr, "%s", getwar(CZ_DBNOTEXIST));
    if (create_empty_db() == CZ_DBFATALERROR) {
      fprintf(stderr, "%s", geterr(CZ_CANTCREATEDB));
      return CZ_CANTCREATEDB;
    }
  }

  /* ------ Konfiguracni soubor ------- */

 /* naalokovani mista s pro config file v HOME adresari */
  if ((pom_str=(char *) malloc((strlen(ADR_CONFIG))+(strlen(CONFIG_HOME))+1)) == NULL) {
    fprintf(stderr ,"%s", geterr(CZ_NOTENOUGHMEM));
    return CZ_NOTENOUGHMEM;
  }
  
  
   /*  pripojeni jmena soubor k ceste do HOME */
   strcpy(pom_str, ADR_CONFIG);
   strcat(pom_str, CONFIG_HOME);
   /* na unixech muze byt konfigurak i jinde nez v $HOME */
#if UNIX   
  if (! (is_conf(pom_str))) { /* nen-li konfigurak v HOME */
      free((void *) pom_str); /* uvolni misto alokovane pro cestu ke konfig v HOME */
      if ((pom_str=(char *) malloc((strlen(SYSCONFDIR))+(strlen(CONFIG_SYS))+1)) == NULL) {
	fprintf(stderr ,"%s", geterr(CZ_NOTENOUGHMEM));
	return CZ_NOTENOUGHMEM;
      }
      strcpy(pom_str, SYSCONFDIR);
      strcat(pom_str, CONFIG_SYS);
  }
#endif

  /* zpracovani dat z konfig souboru */
  if (((fin=fopen(pom_str, "r")) == NULL) && (!opt_q)) fprintf(stderr, "%s \"%s\".\n", getwar(CZ_CANTOPENCONFF), pom_str);
  else    /* podari-li se otevrit konf. soubor */
    if (conf_file(fin) == 1) { /* volame fci z conf.h, ktera vytahne data z konfig. souboru */
      
      if (c_destination_code != NULL) 
        if ((destination_code=ret_cod(c_destination_code)) == -1) { /* pri chybne vlozene hodnote */
          fprintf(stderr, "%s", geterr(CZ_BADOUTPUTCODE_CONF)); /* pis varovani */
	  return CZ_BADOUTPUTCODE_CONF; /* skonci */
	} 
 
      if (c_source_code != NULL) { /* je-li nastaveno v konfiguraci zdrojove kodovani */
        if ((source_code=ret_cod(c_source_code)) == -1) { /* je-li nastaveno spatne */ 
          fprintf(stderr, "%s", geterr(CZ_BADINPUTCODE_CONF));
	  return CZ_BADINPUTCODE_CONF; /* skonci s chybou */
        }
        else opt_s=0;
      }
      
      if (c_pp != NULL) {
        free((void *) pp); pp=NULL; /* uvolni pamet pp, v niz je defaultni pripona */
        if ((pp=(char *) malloc(strlen(c_pp)+1)) == NULL) { /*alokuj dost pameti pro priponu */ 
          fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); /* nedostatek pameti */
	  return 5;
	}
        
        strcpy(pp, c_pp); 
      }
      
      if (c_l_ends != NULL) { 
        if ((l_ends=ret_end(c_l_ends)) == -1) {
	  fprintf(stderr, "%s", geterr(CZ_BADLENDS_CONF)); /* hodnota je spatne zadana */
	  return CZ_BADLENDS_CONF;
	}   
	line_ends=l_ends;
      }
      else line_ends=SAME;

      if ((c_mode < 0) || (c_mode > 3)) {
        fprintf(stderr, "%s", geterr(CZ_BADMODE_CONF));
	return CZ_BADMODE_CONF;
      }
      else mode=c_mode;
      p_num=c_start;

      c_free(); /* uvolneni retezcovych promennych c_* (fce v config.h) */
      fclose(fin);
    }
    else return CZ_CONFFERR;
    
    free((void *) pom_str); /* uvolneni pameti pomocneho retezce */
    pom_str=NULL;
    
    /* --------- konec zpracovani konfiguracniho souboru ----------- */
    
/* -------------------------------------------------------------------- */  

    while((c=getopt(argc, argv, "acCDfhMqSuUvVl:m:p:s:d:o:-")) != -1) {
      switch(c) {
      case 'U': print_dbm_help(); return 0;
      case 'u': opt_u=1; break;
      case 'c': print_format(); return 0; 
      case 'f': opt_f=1; break;
      case 'D': if (dbmshow() == CZ_DBFATALERROR) {
                   fprintf(stderr, "%s", geterr(CZ_CANTOPENDB));
                   return 21;
                }
                else return 0;
      case 'S': print_settings(mode, source_code, destination_code, p_num, pp, l_ends, opt_s);
                return 0;  
      case 'h': print_long_help(); return 0; 
      case 'C': print_codes(); return 0;
      case 'M': print_modes(); return 0;
      case 'q': opt_q=1; break;
      case 'v': opt_v=1; break;
      case 'V': printf("Verze: %s\n%s", VER, AUTHOR); return 0;  /*tiskni verzi programu */
      case 'l': l_ends=ret_end(optarg); /* nastaveni koncu radku */ 
                if ((l_ends != LF) && (l_ends != CRLF)) { /* pri chybne zadane hodnote */
		  fprintf(stderr, "%s", geterr(CZ_BADLINEEND)); /* vypis  */
		  return CZ_BADLINEEND; /* skonci */
		}
		line_ends=l_ends;
		break;
      case 'm': mode=atoi(optarg); /* nastav mod pro vystupni soubor */
                if ((mode > 3) || (mode < 0)) { /* pri chybne zadane hodnote */
		  fprintf(stderr, "%s", geterr(CZ_BADMODE)); /* vypis  */
		  return CZ_BADMODE; /*  */
		}
		break;
      case 'a': opt_a=1; break; /* pouze automaticka detekce */
      case 'd': if ((destination_code=ret_cod(optarg)) == -1) { /* pri chybne vlozene hodnote */
                  fprintf(stderr, "%s", geterr(CZ_BADOUTPUTCODE)); /* pis varovani */
		  return CZ_BADOUTPUTCODE; /* nastav default */
		}   
                break; 
      case 's': if ((source_code=ret_cod(optarg)) == -1) { 
                  fprintf(stderr, "%s", geterr(CZ_BADINPUTCODE));
		  return CZ_BADINPUTCODE;
		} 
		else opt_s=0; /* vypni autodetekci */
                break;
      case 'o': if ((out_fi=(char *) malloc(strlen(optarg)+1)) == NULL) { /*alokuj dost pameti pro jmeno */ 
                  fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); /* nedostatek pameti */
		  return 6;
		} 
		strcpy(out_fi, optarg); /* prekopiruj optarg */ 
		break;
      case 'p': if (mode == 2) { /* zpracuj priponu */ 
                  free((void *) pp); pp=NULL; /* uvolni pamet pp, v niz je defaultni pripona */
		  if ((pp=(char *) malloc(strlen(optarg)+1)) == NULL) { /*alokuj dost pameti pro priponu */ 
                    fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); /* nedostatek pameti */
		    return 5;
		  } 
		  strcpy(pp, optarg); /* prekopiruj optarg */  
                }
		else /* je-li mod jiny nez 2 */
		  if (mode == 3) p_num=atoi(optarg); /* zpracuj startovni cislo */
		  else { fprintf(stderr, "%s", geterr(CZ_CANTSETSUFFIX)); /* spatny mod, nelze nastavit -p */
		         return CZ_CANTSETSUFFIX;
			}
		break;
      }
    } 

/* ######################################################################## */
/* dbmadd */
/* ######################################################################## */
  if (opt_u == 1) {
    if (optind == argc) { fprintf(stderr, "%s", geterr(CZ_INPUTFNEEDED)); CLEAN; return 20; }
    if (opt_s == 1) { fprintf(stderr, "%s", geterr(CZ_INPUTCODENEEDED)); CLEAN; return 22; }
    
    for (i=optind; i!=argc; i++) {
      if ((fin=open_file(argv[i], "r", &prep)) == NULL) fprintf(stderr, " pro cteni.\n");
      else {
        if ((hlp=autodetect(fin)) == CZ_DBFATALERROR) {/* nepodari-li se otevrit databazi */
          fprintf(stderr, "%s", geterr(CZ_CANTOPENDB));
          CLEAN;
          fclose(fin);
          return 21;
        }
        if (fclose(fin) == EOF) { fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTCLOSEF), argv[i]); return CZ_CANTCLOSEF; }
        if ((fin=open_file(argv[i], "r", &prep)) == NULL) fprintf(stderr, " pro cteni.\n");
	else {
          if (hlp < 0) hlp=hlp*(-1);
          c='a'; /* pokud nedojde ke zmene, pridavat se bude */
          if (hlp == 0) { /* jestlize se nezdari autodetekce, tak upozorni */
            printf("cz2cz: Autodetekce nedokazala rozpoznat kodovani souboru \"%s\". Je mozne, ze soubor"
                   " je ulozen v kodovani, ktere neni podporovano, nebo obsahuje jen slova, ktera by nebyla vhodna pro"
                   " pridavani do databaze.\n", argv[i]);
            printf("Prejete si presto vlozit slova ze souboru \"%s\" do databaze (a/n)? ", argv[i]);
	    c=getchar();
	    while (getchar() != '\n'); /* docti buffer do konce a vse zahod */
	  }
          else {
            if ((hlp != source_code) && (hlp != CZ_ASCII)) {
              /* pokud se autodetekce podari a neni stejan jako source_code a nedojde k chybe databaze a neni to 7-bit ASCII */
              printf("cz2cz: Autodetekce odhalila, ze kodovani souboru \"%s\" (%s) se neshoduje s vami zadanou hodnotou."
                     " Nedoporucuje se pridavat slova ze souboru, u ktereho bezpecne neznate zdrojove kodovani - muze to vest"
         	     " k znehodnoceni detekcniho algoritmu.\n", argv[i], ret_charcod(hlp));
	      printf("Prejete si presto vlozit slova ze souboru \"%s\" do databaze (a/n)? ", argv[i]);
	      c=getchar();
              while (getchar() != '\n'); /* docti buffer do konce a vse zahod */ 
	    }
          }
          if ((c == 'a') || (c == 'y')) {
	    if ((db_words=dbmadd(fin, source_code, opt_v)) == CZ_DBFATALERROR) { 
	      fprintf(stderr, "%s", geterr(CZ_CANTOPENDB));
	      CLEAN;
	      fclose(fin);
	      return 21;
	    }
	    printf("cz2cz: Ze souboru %s bylo do databaze pridano %ld slov.\n", argv[i], db_words);
	  }
          fclose(fin);
        }
      }
    }
    CLEAN;
    return 0;
  }

/* ######################################################################## */
/* autodet */
/* ######################################################################## */

  if (opt_a == 1) {
    if (optind == argc) {
      hlp=autodetect(stdin);
      if (hlp == CZ_DBFATALERROR) { fprintf(stderr, "%s", geterr(CZ_CANTOPENDB)); CLEAN; return 21; }
      if (hlp == 0) { CLEAN; fprintf(stderr, "%s", geterr(CZ_DETECTFAILED)); return 0; } 
      if (hlp > 0) printf("cz2cz: Detekovano kodovani %s\n", ret_charcod(hlp));
      else { if (!opt_q) fprintf(stderr, "%s", getwar(CZ_SIMPLEDETECT));
             printf("cz2cz: Detekovano kodovani %s\n", ret_charcod(hlp*(-1)));
      }
    } 
    else {
      for (i=optind; i!=argc; i++)
        if ((fin=open_file(argv[i], "r", &prep)) == NULL) fprintf(stderr, " pro cteni.\n");
	else {
	  hlp=autodetect(fin);
	  if (hlp == CZ_DBFATALERROR) { fprintf(stderr, "%s", geterr(CZ_CANTOPENDB)); CLEAN; return 21; } /* chyba databaze */
	  
	  /*presun na zacatek a test koncu radku */
	  if (fseek(fin, SEEK_SET, 0)) { fprintf(stderr, "%s \"%s\".\n", geterr(CZ_SEEKERROR), argv[i]); }
	  l_ends=autodetect_line_ends(fin); /* detekuj konce radku */
	  
	  /* vypis vysledky */
	  if (hlp == 0) puterr(1, argv[i]); /* detekce neuspesna */
	  if (hlp < 0) {  /* detekce uspesna jednoduchym alg. */
	    printf("cz2cz: Soubor %s ma kodovani %s, %s\n", argv[i], ret_charcod(hlp*(-1)), ret_charend(l_ends)); 
	    if (!opt_q) puterr(2, argv[i]); 
	  }
	  else if (hlp > 0) /* detekce uspesna databazovym algoritmem */ 
	         printf("cz2cz: Soubor %s ma kodovani %s, %s\n", argv[i], ret_charcod(hlp), ret_charend(l_ends));
          if (fclose(fin) == EOF) { fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTCLOSEF), argv[i]); return CZ_CANTCLOSEF; }
        }
    }
    CLEAN;
    return 0;
  }

/* ######################################################################### */
/* mode 0 */
/* ######################################################################### */

  if (mode == 0) {
    if (argc == optind) { /* neni-li zadan vstupni soubor */
      if (readstdin2tmp() == 0) {
        if (opt_s == 1) { /* proved autodetekci */
	  if ((source_code=autod(tmp_nam, CZ_STDIN)) <= -1) { CLEAN; return -1; }
	  if (opt_v == 1) printf("cz2cz: Detekovano kodovani %s.\n", ret_charcod(source_code));
	}
	/*
	if ((source_code == destination_code) && (line_ends == l_ends)) {
          if (opt_v == 1) printf("%s", getwar(CZ_SAMECODES));
	  CLEAN;
	  remove(tmp_nam);
	  return 0;
        }*/
	
	if ((fin=fopen(tmp_nam, "r")) == NULL) {
	  fprintf(stderr, "%s temp pro cteni.", geterr(CZ_CANTOPENF));
	  CLEAN;
	  remove(tmp_nam);
	  return 4;
	}
        if (out_fi == NULL) { /* neni-li zadan vystupni soubor ani vstupni soubor */
          switch (recode(fin, stdout, source_code, destination_code, l_ends)) {
	    case 1: fprintf(stderr, "%s", geterr(CZ_STDOUTERR)); CLEAN; fclose(fin); remove(tmp_nam); return 7;  
	    case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); CLEAN; fclose(fin); remove(tmp_nam); return 8;
	    case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); CLEAN; fclose(fin); remove(tmp_nam); return 11;
            default: CLEAN; fclose(fin); remove(tmp_nam); return 0;
          }	
        }
        else { 
          if (ret_dir(out_fi) == 0) { fprintf(stderr, "%s", geterr(CZ_INPUTFDIRERR)); remove(tmp_nam); CLEAN; return 12;} /* je-li vstupni soubor adr. */ 
          if ((fout=open_file(out_fi, "wb", &prep)) == NULL) {
	     if (prep == 0) fprintf(stderr, " pro zapis.\n"); /* nedoslo-li na dotazovani zda soubor prepsat */ 
	     CLEAN;
	     remove(tmp_nam);
	     return 4;
	  }
	  switch (recode(fin, fout, source_code, destination_code, l_ends)) {
	    case 1: fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTWRITETOF), out_fi); fclose(fout); fclose(fin); remove(tmp_nam); CLEAN; return 7;  
	    case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fout); fclose(fin); remove(tmp_nam); CLEAN; return 8;
	    case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); fclose(fout); fclose(fin); remove(tmp_nam); CLEAN; return 11;
            default: if (opt_v == 1) printf("cz2cz: Text preveden a zapsan do souboru \"%s\".\n", out_fi); 
                   fclose(fout); CLEAN; return 0;
          }
        }	
      } /* konec readstdin2tmp */	
    }  
    else { /* je-li zadan vstupni soubor */
      if (ret_dir(argv[optind]) == 0) { fprintf(stderr, "%s", geterr(CZ_OUTPUTFDIRERR)); CLEAN; return 13; }
      
      if (opt_s == 1) { /* proved autodetekci */
	if ((source_code=autod(argv[optind], 0)) <= -1) { CLEAN; return -1; }
	if (opt_v == 1) printf("cz2cz: Detekovano kodovani %s souboru \"%s\".\n", ret_charcod(source_code), argv[optind]);
      }
/*
      if ((source_code == destination_code) && (line_ends == l_ends)) {
        if (opt_v == 1) printf("%s: %s", argv[optind], getwar(CZ_SAMECODES));
	CLEAN;
	return 0;
      }
*/      
      if ((fin=open_file(argv[optind], "rb", &prep)) == NULL) {
        fprintf(stderr, " pro cteni.\n");
	CLEAN;
	return 4; 
      }      
      if (opt_v == 1) printf("cz2cz: Otevreni vstupniho souboru \"%s\" vporadku.\n", argv[optind]); 
      if (out_fi == NULL)/* neni-li zadan vystupni soubor a vstupni ano */
        switch (recode(fin, stdout, source_code, destination_code, l_ends)) {
	  case 1: fprintf(stderr, "%s", geterr(CZ_STDOUTERR)); fclose(fin); CLEAN; return 7;  
	  case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fin); CLEAN; return 8;
	  case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); fclose(fin); CLEAN; return 11;
          default: fclose(fin); CLEAN; return 0;
        }
      else { /* je-li zadan vystupni soubor i vstupni */
        if (ret_dir(out_fi) == 0) { fprintf(stderr, "%s", geterr(CZ_INPUTFDIRERR)); CLEAN; return 12; } 
        if ((fout=open_file(out_fi, "wb", &prep)) == NULL) {
	   if (prep == 0) fprintf(stderr, " pro zapis.\n"); /* nedoslo-li k dotazovani na prepis soubor */
	   fclose(fin);
	   CLEAN; 
	   return 4;
	}
	switch (recode(fin, fout, source_code, destination_code, l_ends)) {
	  case 1: fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTWRITETOF), out_fi); fclose(fout); fclose(fin); CLEAN; return 7;  
	  case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fout); fclose(fin); CLEAN; return 8;
          case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); fclose(fout); fclose(fin); CLEAN; return 11;
          default: if (opt_v == 1) printf("cz2cz: Text preveden a zapsan do souboru \"%s\".\n", out_fi);
                   fclose(fout); fclose(fin); CLEAN; return 0;
        }
      }    
    }
  } 

/* ######################################################################### */
/* mode 1 */ 
/* ######################################################################### */    

  if (mode == 1) { 
    if (argc == optind) { 
      fprintf(stderr, "%s", geterr(CZ_NOINPUTF));
      CLEAN;
      return 10;
    }
    if (out_fi == NULL) {        
      for (i=optind; i!=argc; i++) { /* postupne zpracuj vstupni soubory */
        if (ret_dir(argv[i]) == 0) { fprintf(stderr, "%s", geterr(CZ_OUTPUTFDIRERR)); CLEAN; return 13; }
        
	if (opt_s == 1) { /* proved autodetekci */
	  if ((source_code=autod(argv[i], 0)) == -1) { CLEAN; return -1; }
          if (source_code == -2) continue;
	  if (opt_v == 1) printf("cz2cz: Detekovano kodovani %s souboru \"%s\".\n", ret_charcod(source_code), argv[i]);
        }
	/*
        if ((source_code == destination_code) && (line_ends == l_ends)) {
          if (opt_v == 1) printf("%s: %s", argv[i], getwar(CZ_SAMECODES));
	  continue;
      	}*/
        if ((fin=open_file(argv[i], "rb", &prep)) == NULL) {
          fprintf(stderr, " pro cteni.\n");
	  continue; /* pokracuj na zpracovani dalsiho souboru */
        }
	
        if (opt_v == 1) printf("Soubor \"%s\" otevren v poradku pro cteni.\n", argv[i]);
	
        tmp_nam=tmpnam(NULL);
        if ((fout=fopen(tmp_nam, "wb")) == NULL) { /* otevri pomocny soubor */
	  fprintf(stderr, "%s", geterr(CZ_CANTPREPTEMP));
	  fclose(fin); 
	  continue;
        }
        switch (recode(fin, fout, source_code, destination_code, l_ends)) {
	  case 1: fprintf(stderr, "%s temp.\n", geterr(CZ_CANTWRITETOF)); fclose(fout); fclose(fin); remove(tmp_nam); CLEAN; continue;  
	  case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fout); fclose(fin); remove(tmp_nam);  
	           if (opt_s == 1) continue; /* priznak pro pokracovani, bylo-li kodovani souboru zjisteno autodetekci */
		   else { CLEAN; return 8; } /* bylo-li kodovani zadano uzivatelem ani neni treba dalsi soubory zkouset */
          case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); fclose(fout); fclose(fin); remove(tmp_nam); CLEAN; return 11;
          default: fclose(fout); fclose(fin); break; /* zavri soubory a pokracuj na dalsi */
        }	
	
        /* priprav prekopirovani z temp souboru do puvodniho vstupu */
        if ((fin=fopen(tmp_nam, "r")) == NULL) {
	  fprintf(stderr, "%s", geterr(CZ_CANTOPENTEMP));
	  fclose(fin);
	  remove(tmp_nam);
	  continue;
        }
        if ((fout=fopen(argv[i],"w")) == NULL) {
	  fprintf(stderr, "%s \"%s\" pro zapis.\n", geterr(CZ_CANTOPENF), argv[i]);
          fclose(fin);
	  remove(tmp_nam);
	  continue;
        }
	while ((c=getc(fin)) != EOF) putc(c, fout);
        if (opt_v == 1) printf("cz2cz: Text uspesne preveden a zapsan do souboru \"%s\".\n", argv[i]);
	fclose(fin); fclose(fout);
	remove(tmp_nam);	
      }   /* ukonci for cyklus */	
    }
    else { /* neni-li out_fi prazdne */ 
     if (ret_dir(out_fi) == 0) { /* je-li out_fi adresarem */
        for (i=optind; i!=argc; i++) {
	  /* nyni otevreme vstupni a vystupni soubor */
          if (ret_dir(argv[i]) == 0) { fprintf(stderr, "%s", geterr(CZ_OUTPUTFDIRERR)); CLEAN; return 13; }
          
	  if (opt_s == 1) { /* proved autodetekci */
	    if ((source_code=autod(argv[i], 0)) == -1) { CLEAN; return -1; }
	    if (source_code == -2) continue;
            if (opt_v == 1) printf("Detekovano kodovani %s souboru \"%s\".\n", ret_charcod(source_code), argv[i]);
          }/*
          if ((source_code == destination_code) && (line_ends == l_ends)) {
            if (opt_v == 1) printf("%s: %s", argv[i], getwar(CZ_SAMECODES));
	    continue;
      	  } */
	
	  if ((fin=open_file(argv[i], "rb", &prep)) == NULL) { 
	    fprintf(stderr, " pro cteni.\n");
            continue;      
	  }
          if (opt_v == 1) printf("cz2cz: Soubor \"%s\" otevren v poradku pro cteni.\n", argv[i]);
	  if (ret_path(out_fi, argv[i]) == NULL) return 11; /* neuspech cesty k adres. */
	  if (ret_dir(ret_path(out_fi, argv[i])) == 0) { fprintf(stderr, "%s", geterr(CZ_INPUTFDIRERR)); continue; } /* je-li vystupni soubor adr. */
          if ((fout=open_file(ret_path(out_fi, argv[i]), "wb", &prep)) == NULL) {
	    if (prep == 0) fprintf(stderr, " pro zapis.\n"); /* nedoslo-li k dotazovani na prepis soubor */
	    fclose(fin);
	    continue;
	  }
          switch (recode(fin, fout, source_code, destination_code, l_ends)) {
	    case 1: fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTWRITETOF), out_fi); fclose(fout); fclose(fin); CLEAN; continue;  
	    case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fout); fclose(fin); 
	             if (opt_s == 1) continue;
		     else { CLEAN;  return 8; }
            case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); fclose(fout); fclose(fin); CLEAN; return 11;
            default: if (opt_v == 1) printf("cz2cz: Text uspesne preveden a zapsan do souboru \"%s\".\n", ret_path(out_fi, argv[i]));
                     fclose(fout); fclose(fin); break; /* zavri soubory a pokracuj na dalsi */
          }	
        } /* konec for cyklu */
      } /* konec testu na adresar */
      else { /* neni-li out_fi adr. */
        if (ret_dir(out_fi) == -1) fprintf(stderr, "%s\"%s\" neexistuje.\n", geterr(CZ_DIR), out_fi);
	else fprintf(stderr, "%s", geterr(CZ_DIRNEEDED));
        CLEAN;
        return 16;
      }
    } /* konec else neni-li out_fi prazdne */
    CLEAN; 
    return 0; 
  } /* konec mode 1 */

/* ######################################################################### */
/* mode 2 */
/* ######################################################################### */

  if (mode == 2) {
    if (strlen(pp) == 0) {
      fprintf(stderr, "%s", geterr(CZ_EMPTYSUFFIX));
      CLEAN;
      return 17;
    } 
    if (optind == argc) {
      fprintf(stderr, "%s", geterr(CZ_NOINPUTF));
      CLEAN;
      return 10;
    }
    if (out_fi == NULL) { /* neni-li zadan adresar */      
      for (i=optind; i!=argc; i++) {
        if (ret_dir(argv[i]) == 0) { fprintf(stderr, "%s", geterr(CZ_OUTPUTFDIRERR)); CLEAN; return 13; }
        
	if (opt_s == 1) { /* proved autodetekci */
	  if ((source_code=autod(argv[i], 0)) == -1) { CLEAN; return -1; }
          if (source_code == -2) continue;
	  if (opt_v == 1) printf("Detekovano kodovani %s souboru \"%s\".\n", ret_charcod(source_code), argv[i]);
        }
	/*
        if ((source_code == destination_code) && (line_ends == l_ends)) {
          if (opt_v == 1) printf("%s: %s", argv[i], getwar(CZ_SAMECODES));
	  continue;
      	} */
	
	if ((fin=open_file(argv[i], "rb", &prep)) == NULL) { 
	  fprintf(stderr, " pro cteni.\n");
          continue;    
        }
        if (opt_v == 1) printf("cz2cz: Soubor \"%s\" otevren vporadku pro cteni.\n", argv[i]);
	/* priprava mista pro vystup s priponou, pouziji out_fi */
	if ((out_fi=(char *) malloc(strlen(argv[i])+strlen(pp)+1)) == NULL) {
	  fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM));
	  fclose(fin); /* zavri vstup */
	  CLEAN;
	  return 11;
        }
	strcpy(out_fi, argv[i]);
	strcat(out_fi, pp);
	if ((fout=open_file(out_fi, "wb", &prep)) == NULL) {
	  if (prep == 0) fprintf(stderr, " pro zapis.\n"); 
	  continue; /* nepodari-li se vytvorit soubor pro zapis, pokracuj na zprac. dalsich soub. */
        }
        switch (recode(fin, fout, source_code, destination_code, l_ends)) {
	  case 1: fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTWRITETOF), out_fi); fclose(fout); fclose(fin); CLEAN; continue;  
	  case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fout); fclose(fin); 
	           if (opt_s == 1) continue;
		   else { CLEAN;  return 8; }
	  case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); CLEAN; return 11;
          default: if (opt_v == 1) printf("cz2cz: Text uspesne preveden a zapsan do souboru \"%s\".\n", out_fi);
                   fclose(fout); fclose(fin); break; /* zavri soubory a pokracuj na dalsi */
        }
      }
    }
    else { /* je-li zadan adresar */
      if (ret_dir(out_fi) == 0) { /* je-li out_fi adresarem */
        for (i=optind; i!=argc; i++) {
	  if (ret_dir(argv[i]) == 0) { fprintf(stderr, "%s", geterr(CZ_OUTPUTFDIRERR)); CLEAN; return 13; }
          
	  if (opt_s == 1) { /* proved autodetekci */
	    if ((source_code=autod(argv[i], 0)) == -1) { CLEAN; return -1; }
            if (source_code == -2) continue;
	    if (opt_v == 1) printf("Detekovano kodovani %s souboru \"%s\".\n", ret_charcod(source_code), argv[i]);
          }
	  /*
          if ((source_code == destination_code) && (line_ends == l_ends)) {
            if (opt_v == 1) printf("%s: %s", argv[i], getwar(CZ_SAMECODES));
	    continue;
      	  } */
	  
	  if ((fin=open_file(argv[i], "rb", &prep)) == NULL) {
	    fprintf(stderr, " pro cteni.\n");
            continue;  
	  }	 
          if (opt_v == 1) printf("cz2cz: Soubor \"%s\" otevren vporadku pro cteni.\n", argv[i]);
          /* priprava promenne na jemo pro vystup s priponou */
	  if ((pom_str=(char *) malloc(strlen(argv[i])+strlen(pp)+1)) == NULL) {
	    fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM));
	    fclose(fin); /* zavri vstup */
	    CLEAN;
	    return 11;
          }
	  strcpy(pom_str, argv[i]);
	  strcat(pom_str, pp);
	  
	  if (ret_path(out_fi, pom_str) == NULL) return 11; /* neuspech cesty k adres. */
	  if (ret_dir(ret_path(out_fi, pom_str)) == 0) { fprintf(stderr, "%s", geterr(CZ_INPUTFDIRERR)); continue; }
	  
	  if ((fout=open_file(ret_path(out_fi, pom_str), "wb", &prep)) == NULL) {
	    if (prep == 0) fprintf(stderr, " pro zapis.\n"); /* nedoslo-li k dotazovani na prepis soubor */
	    free((void *) pom_str); pom_str=NULL;
	    fclose(fin);
	    continue;
	  }
          switch (recode(fin, fout, source_code, destination_code, l_ends)) {
	    case 1: fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTWRITETOF), out_fi); fclose(fout); fclose(fin); CLEAN; continue;  
	    case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fout); fclose(fin); 
	             if (opt_s == 1) continue;
		     else { CLEAN;  return 8; }
            case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); fclose(fout); fclose(fin); CLEAN; return 11;
            default: if (opt_v == 1) printf("cz2cz: Text uspesne preveden a zapsan do souboru \"%s\".\n", ret_path(out_fi, pom_str));
                     free((void *) pom_str); pom_str=NULL;
                     fclose(fout); fclose(fin); break; /* zavri soubory a pokracuj na dalsi */
          }	
        }/* konec for cyklu */
      } /* konec testu na spravnost adr. */
      else { /* neni-li out_fi adresarem */
        if (ret_dir(out_fi) == -1) fprintf(stderr, "%s\"%s\" neexistuje.\n", geterr(CZ_DIR), out_fi);
        else fprintf(stderr, "%s", geterr(CZ_DIRNEEDED));
        CLEAN;
        return 16;
      }
    } /* konec else je-li zadan adresar */  
    CLEAN;
    return 0; 
  } /* konec mode 2 */
 
/* ######################################################################### */
/* mode 3 */
/* ######################################################################### */

  if (mode == 3) {
    if (optind == argc) {
      fprintf(stderr, "%s", geterr(CZ_NOINPUTF));
      CLEAN;
      return 10;
    }
    if (out_fi == NULL) { /* neni-li zadan adresar */      
      for (i=optind; i!=argc; i++) {
        if (ret_dir(argv[i]) == 0) { fprintf(stderr, "%s", geterr(CZ_OUTPUTFDIRERR)); CLEAN; return 13; }
        
	if (opt_s == 1) { /* proved autodetekci */
	  if ((source_code=autod(argv[i], 0)) == -1) { CLEAN; return -1; }
          if (source_code == -2) continue;
          if (opt_v == 1) printf("Detekovano kodovani %s souboru \"%s\".\n", ret_charcod(source_code), argv[i]);
        }
	/*
        if ((source_code == destination_code) && (line_ends == l_ends)) {
          if (opt_v == 1) printf("%s: %s", argv[i], getwar(CZ_SAMECODES));
	  continue;
      	} 
	*/
	
	if ((fin=open_file(argv[i], "rb", &prep)) == NULL) { 
	  fprintf(stderr, " pro cteni.\n");
          continue;    
        }
        if (opt_v == 1) printf("cz2cz: Soubor \"%s\" otevren vporadku pro cteni.\n", argv[i]);
	/* priprava mista pro vystup s cislem, pouziji out_fi */
	if ((out_fi=(char *) malloc(strlen(argv[i])+MAX+1)) == NULL) { /* MAX je pro delku cisla prevedeneho na retezec */
	  fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM));
	  fclose(fin); /* zavri vstup */
	  CLEAN;
	  return 11;
        }
        sprintf(out_fi,"%s%lu", argv[i], p_num);
	
	if ((fout=open_file(out_fi, "wb", &prep)) == NULL) {
	  if (prep == 0) fprintf(stderr, " pro zapis.\n"); 
	  continue; /* nepodari-li se vytvorit soubor pro zapis, pokracuj na zprac. dalsich soub. */
        }
        switch (recode(fin, fout, source_code, destination_code, l_ends)) {
	  case 1: fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTWRITETOF), out_fi); fclose(fout); fclose(fin); CLEAN; continue;  
	  case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fout); fclose(fin); 
	           if (opt_s == 1) continue;
		   else { CLEAN; return 8; }
	  case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); CLEAN; return 11;
          default: if (opt_v == 1) printf("cz2cz: Text uspesne preveden a zapsan do souboru \"%s\".\n", out_fi);
                   fclose(fout); fclose(fin); break; /* zavri soubory a pokracuj na dalsi */
        }
	p_num++;
      }
    }
    else { /* je-li zadan adresar */
      if (ret_dir(out_fi) == 0) { /* je-li out_fi adresarem */
        for (i=optind; i!=argc; i++) {
	  if (ret_dir(argv[i]) == 0) { fprintf(stderr, "%s", geterr(CZ_OUTPUTFDIRERR)); CLEAN; return 13; }
          
	  if (opt_s == 1) { /* proved autodetekci */
	    if ((source_code=autod(argv[i], 0)) == -1) { CLEAN; return -1; }
            if (source_code == -2) continue;
	    if (opt_v == 1) printf("Detekovano kodovani %s souboru \"%s\".\n", ret_charcod(source_code), argv[i]);
          }
	  /*
          if ((source_code == destination_code) && (line_ends == l_ends)) {
            if (opt_v == 1) printf("%s: %s", argv[i], getwar(CZ_SAMECODES));
	    continue;
      	  } 
	  */
	  
	  if ((fin=open_file(argv[i], "rb", &prep)) == NULL) {
	    fprintf(stderr, " pro cteni.\n");
            continue;  
	  }	 
          if (opt_v == 1) printf("cz2cz: Soubor \"%s\" otevren vporadku pro cteni.\n", argv[i]);
          /* priprava promenne na jmeno pro vystup s priponou */
	  if ((pom_str=(char *) malloc(strlen(argv[i])+MAX+1)) == NULL) {
	    fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM));
	    fclose(fin); /* zavri vstup */
	    CLEAN;
	    return 11;
          }
	  sprintf(pom_str, "%s%lu", argv[i], p_num);
	  
	  if (ret_path(out_fi, pom_str) == NULL) return 11; /* neuspech cesty k adres. */
	  if (ret_dir(ret_path(out_fi, pom_str)) == 0) { fprintf(stderr, "%s", geterr(CZ_INPUTFDIRERR)); continue; }
	  
	  if ((fout=open_file(ret_path(out_fi, pom_str), "wb", &prep)) == NULL) {
	    if (prep == 0) fprintf(stderr, " pro zapis.\n"); /* nedoslo-li k dotazovani na prepis soubor */
	    free((void *) pom_str); pom_str=NULL;
	    fclose(fin);
	    continue;
	  }
          switch (recode(fin, fout, source_code, destination_code, l_ends)) {
	    case 1: fprintf(stderr, "%s \"%s\".\n", geterr(CZ_CANTWRITETOF), out_fi); fclose(fout); fclose(fin); CLEAN; continue;  
	    case -1: fprintf(stderr, "%s", geterr(CZ_CODENOTSUPP)); fclose(fout); fclose(fin); 
	             if (opt_s == 1) continue;
		     else { CLEAN; return 8; }
	    case -2: fprintf(stderr, "%s", geterr(CZ_NOTENOUGHMEM)); fclose(fout); fclose(fin); CLEAN; return 11;
            default: if (opt_v == 1) printf("cz2cz: Text uspesne preveden a zapsan do souboru \"%s\".\n", ret_path(out_fi, pom_str));
                     fclose(fout); fclose(fin); break; /* zavri soubory a pokracuj na dalsi */
          }
	  p_num++;	
        }/* konec for cyklu */
      } /* konec testu na spravnost adr. */
      else { /* neni-li out_fi adresarem */
        if (ret_dir(out_fi) == -1) fprintf(stderr, "%s\"%s\" neexistuje.\n", geterr(CZ_DIR), out_fi);
        else fprintf(stderr, "%s", geterr(CZ_DIRNEEDED));
        CLEAN;
        return 16;
      }
    } /* konec else je-li zadan adresar */  
    CLEAN;
    return 0; 
  } /* konec mode 3 */

  CLEAN;
  return 0;       
}






















































































































