/*
 * cz2cz tools
 * Copyright (C) 2005 Vlastimil Krejcir
 *
 * This file is part of cz2cz.
 *
 * cz2cz is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


#include<curses.h>
#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include<signal.h>
#include<ctype.h>
#if UNIX
 #include<sys/stat.h>
 #include<sys/types.h>
 #include<unctrl.h> 
#endif
#if DOS
 #include<sys\stat.h>
 #include<sys\types.h>
#endif
#include<unistd.h>

#include"autodet.h"
#include"tables.h"
#include"codes.h"
#include"version.h"

#define CZ_DBFATALERROR -127
#define UNDO_LIM 50
#define CUT_NAME (COLS-35)
#define CUT_NAME_BASIC (COLS-48)
#define CUT_NAME_SHORT (COLS-60)
#define MAX_FILE_PATH 4096
#define CR 13
#define LF 10
#define COR move(LINES-1, COLS-1)
#if UNIX
 #define DEST 1 /* v unixech se pouziva cestina iso-8859-2 */
#endif
#if DOS
 #define DEST 3 /* windows pouziva cestinu PCLatin */
 #define TABSIZE 6
#endif

typedef struct list { long val;
                      struct list *nxt;
		      struct list *prv;
		    } LIST;
		    
typedef struct undo { long pstn;
                      unsigned char ch;
                      struct undo *nxt;
		      struct undo *prv;
		    } UNDO;

unsigned char *file; /* pracovni buffer */
FILE *frw, *tmp;
int flag_tmp=1, flag_crlf=0;
long l_file, ind; /* jak velky buffer */
int source_code, destination_code, l_ends;
char name[MAX_FILE_PATH], hlp_name[MAX_FILE_PATH]; /* jmeno souboru, pomocne jmeno */
int modified=0; /* byl soubor modifikovan? */
long position=0; /* kde jsme v souboru */
LIST *p_first, *p_act, *p_hlp; /* pro zobrazovani obrazovek */
long next; /* pro zobrazovani */
long relpos=0; /* relativni pozice pro doplnovani */  
int *wherex, *wherey; /* ke kazdemu znaku urci jeho pozici na obrazovce */
int cycle_flag; /* pokud mam vsechny mozne znaky doplneny, neni kam skocit */
long page; /* cislovani stran */
UNDO *u_first, *u_act, *u_hlp, *u_last; /* pro undo - neomezeno poctem */

/* position - znak na prvnim miste stranky terminalu
   next - nasledujici prvni znak, ktery se bude vypisovat (vraci fce show()) */

/* ovladac signalu pri ukonceni */ 
static void finish(int sig)
{ 
  endwin();
  exit(0);
}             
/* konec ovladace signalu */

char *cutname(const char *f_name, int len) /* orizne delku cesty k souboru 'f_name' na 'len' znaku */
{ char hlp[MAX_FILE_PATH+6];
  char *underch;
  int abs_path=0;
  char pom;

#if UNIX
  if (f_name[0] == '/') abs_path=1;
#endif
#if DOS
  if (f_name[0] == '\\') abs_path=1;
#endif
  strcpy(hlp_name, f_name);
  strcat(hlp, f_name);
  if (strlen(hlp_name) > len) {  /* je-li cesta delsi nez len */ 
#if UNIX
    if (strchr(hlp_name, '/') != NULL) {
#endif
#if DOS
    if (strchr(hlp_name, '\\') != NULL) {
#endif
      do {
        pom=hlp_name[0];
	hlp_name[0]='?';
#if UNIX
        if ((underch=strchr(hlp_name, '/')) != NULL) {
#endif
#if DOS
	if ((underch=strchr(hlp_name, '\\')) != NULL) {
#endif
          strcpy(hlp_name, underch);
	  abs_path=0;
	}
	else hlp_name[0]=pom;
      }while ((strlen(hlp_name) > (len-3)) && (underch != NULL)); /* dokud neni cesta (vcetne tri tecek) kratsi nez len, 
                                                                    nebo dojdu-li na konec retezce */
#if UNIX    
      if (hlp_name == NULL) strcpy(hlp_name, strrchr(hlp, '/')); /* dojdu-li az na konec tak zkopiruji vse vcetne posledniho lomitka */
#endif
#if DOS
      if (hlp_name == NULL) strcpy(hlp_name, strrchr(hlp, '\\'));
#endif
      if (abs_path == 0) { /* pred lomitko pridam "..." pokud neni cesta absolutni*/
        strcpy(hlp, "...");  
        strcat(hlp, hlp_name);
        strcpy(hlp_name, hlp);
      }
      if (strlen(hlp_name) > len) { /* je-li retezec delsi nez len */
        strcpy(hlp, "");
        strncpy(hlp, hlp_name, len-4); /* orizni jmeno souboru */
	hlp[len-4]='\0'; /* zakonci */
	strcat(hlp, "...");    /* pridej "..." */
	strcpy(hlp_name, hlp);
      }
    }
    else { /* orizni jmeno souboru a ukonci ho trojteckou neobsahuje-li cesta '/'*/
      strncpy(hlp_name, f_name, len-4);
      hlp_name[len-4]='\0';
      strcat(hlp_name, "...");
      addstr(hlp_name);
    }
  }    
  return hlp_name; 
}

int autod(const char *f) /* autodetekce pro prevody */
{ FILE *ff;
  int h;

  if ((ff=fopen(f, "r")) == NULL) {
    fprintf(stderr, "Soubor nelze otevrit pro cteni (autodetekce).\n");
    return -1;
  }
  else {
    h=autodetect(ff);
    if (fclose(ff) == EOF) { fprintf(stderr, "xcz2cz: Nemohu uzavrit soubor %s.\n", f); return -1; }
    
    if (h == CZ_DBFATALERROR) { fprintf(stderr, "xcz2cz: Chyba databaze.\n"); return -1; } /* chyba databaze */
    if (h == 0) { /* detekce neuspesna */
      fprintf(stderr, "xcz2cz: Autodetekce neuspesna - nastavte si prosim kodovani v programu.\n");
      fprintf(stderr, "xcz2cz: Nastavuji ISO-8859-2.\n");
      fprintf(stderr, "xcz2cz: Stisknete Enter pro pokracovani ...");
      getchar(); 
      return -1;
    }
    if (h < 0) {  /* detekce uspesna jednoduchym alg. */
      return h*(-1);
    }
    else {
      if (h > 0) return h; } /* detekce uspesna databazovym algoritmem */

  }
  return -1;
}               

int xautod(char *f) /* autodetekce pro prevody */
{ FILE *ff;
  int h, i;

  if ((ff=fopen(f, "r")) == NULL) {
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Nemohu otevrit soubor \"");
    addstr(cutname(f, CUT_NAME));
    addstr("\". !!!");
    attrset(A_NORMAL);
    COR;
    return -1;
  }
  
  h=autodetect(ff);
  if (fclose(ff) == EOF) { 
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Nemohu uzavrit soubor \"");
    addstr(cutname(f, CUT_NAME));
    addstr("\". !!!");
    attrset(A_NORMAL);
    return -1;
  }
  
  if (h == CZ_DBFATALERROR) { 
    for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Chyba databaze pro autodetekci. !!!");
    attrset(A_NORMAL);
    return CZ_DBFATALERROR;
  } /* chyba databaze */
  if (h == 0) { /* detekce neuspesna */
    move(LINES-2, 1);
    attrset(A_STANDOUT);
    addstr("Autodetekce kodovani souboru \"");
    addstr(cutname(f, CUT_NAME_BASIC));
    addstr("\" neuspesna.");
    move(LINES-1, 1);
    addstr("Nastavte si prosim kodovani manualne.");
    attrset(A_NORMAL);
    COR;    
    getch();
    return 0;
  }
  if (h < 0) {  /* detekce uspesna jednoduchym alg. */
    return h*(-1);
  }
  else {
    if (h > 0) return h; 
  } /* detekce uspesna databazovym algoritmem */

  return -1;
}               

/*
char *ret_charcod(int code)
{ switch (code) {
  case 1: return "ISO-8859-2";
  case 2: return "Win-1250";
  case 3: return "PCLatin";
  case 4: return "Kamenickych";
  case 5: return "7-bit ASCII";
  case 6: return "postfixove sprezky";
  case 7: return "prefixove sprezky";
  case 8: return "TeX konvence";
  default: return "nedef.";
  }
}
*/

int my_getstr(char *str) /* nacti retezec + cekej na pripadne CTRL+U */
{ char key;
  int index=0;  
  int x, y;
  
  getyx(stdscr, y, x);
  move(y, x);
  while ((key=getch()) != CR) {
    if (strcmp(unctrl(key), "^U") == 0) return -1; /* ^C */
    if ((key == '\b') || (key == '\a')) { /* backspace */
      if (index > 0) {
        index--;
	if (x == 0) { 
	  if (y == 0) { 
	    move(LINES-1, COLS-1);
	    addch(' '); 
	    y=LINES-1;
	  } 
	  else { 
	    move(y-1, COLS-1);
	    addch(' '); 
	    y--;
	  } 
	  x=COLS-1;
	}
	else {
	  move(y, x-1);
	  addch(' '); 
	  x--;
	}
      }
    }
    else {
      if (index > (MAX_FILE_PATH-1)) {
        move(y, x+1);
	addch(' ');
	move(y, x);
	addch(' ');
        continue; /* maximalni delka cesty je omezena */
      }
      str[index]=key;
      index++;
      x++;
      if (x > (COLS-1)) { 
        if (y == (LINES-1)) y=0;
        else y++; 
	x=0; }
    }
    move(y, x);
  }
  str[index]='\0';
  
  return 0;
} 

int fil_open(char *n) /* otevreni, prevod do isolf a nacteni souboru */
{  int i, hlp; 
   unsigned char *new_file;
   unsigned char c;
   int new_source_code, new_crlf; /* pro obnovu po chybe */
   unsigned int new_l_file;
  
   if ((frw=fopen(n, "rb")) == NULL) {
      for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
      move(LINES-1, 1);
      attrset(A_STANDOUT);
      addstr("!!! Nemohu otevrit soubor \"");
      addstr(cutname(n, CUT_NAME));
      addstr("\". !!!");
      attrset(A_NORMAL);
      COR;
      getch();
      return -1;
    }
    hlp=0;
    
    new_l_file=0;
    new_crlf=0;
    while (fread(&c, sizeof(c), 1, frw)) {
      if ((new_crlf == 0) && (hlp == CR) && (c == LF)) new_crlf=1;
      hlp=c;
      if (c != CR) new_l_file++;
    }
    
    if ((fseek(frw, 0, SEEK_SET)) != 0) { 
      for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
      move(LINES-1, 0);
      attrset(A_STANDOUT);
      addstr("!!! Chyba pri presunu v souboru \"");
      addstr(cutname(n, CUT_NAME));
      addstr("\". !!!");
      attrset(A_NORMAL);
      COR;
      getch();
      return -1;
    }
    
    hlp=xautod(n);
    if (hlp > 0) new_source_code=hlp;
    else { if (hlp == 0) new_source_code=1; /* detekce se nezdarila, otevri soubor, jako by byl iso */
           else { getch(); return -1; }
	 }
    
    flag_tmp=1; /* nastavim, ze soubor tmp nepouziji */
    if ((new_source_code != DEST) && (new_source_code <= 5)) {
      if ((tmp=tmpfile()) == NULL) {   
        for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
        move(LINES-1, 0);
	attrset(A_STANDOUT);
        addstr("!!! Nemohu otevrit temp soubor !!!");
	attrset(A_NORMAL);
	COR;
        getch();
        return -1;
      } 
      
      if (recode(frw, tmp, new_source_code, DEST, 0) != 0) { /* preved na iso a lf */
        for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
        move(LINES-1, 0);
	attrset(A_STANDOUT);
        addstr("!!! Chyba pri prevodu souboru \"");
        addstr(cutname(n, CUT_NAME));
        addstr("\". !!!");
	attrset(A_NORMAL);
	COR;
        getch();
	return -1;  
      }
      
      flag_tmp=0; /* soubor tmp musel byt otevren */
    }
    fclose(frw);

    if (flag_tmp) { /* soubor tmp nemusi byt pouzit, zdrojovy soubor je ISO nebo ASCII */
      if ((frw=fopen(n, "rb")) == NULL) {
        move(LINES-1, 0);
        attrset(A_STANDOUT);
	addstr("!!! Nemohu otevrit soubor1 \"");
        addstr(cutname(n, CUT_NAME));
        addstr("\" pro cteni. !!!");
        attrset(A_NORMAL);
	COR;
        getch();
        return -1;
      }
    }
    else { /* byl pouzit tmp tak se v nem presun na zacatek a priprav se na cteni */
      if ((fseek(tmp, 0, SEEK_SET)) != 0) { 
        for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
        move(LINES-1, 0);
        attrset(A_STANDOUT);
        addstr("!!! Chyba pri presunu v temp souboru. !!!");
        attrset(A_NORMAL);
        COR;
        getch();
        return -1;
      }
    }  
    
    /* nalokuji pomocny buffer */
    if ((new_file=(unsigned char *) malloc(new_l_file+1)) == NULL) {
      for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
      move(LINES-1, 0);
      attrset(A_STANDOUT);
      addstr("!!! Soubor \"");
      addstr(cutname(n, CUT_NAME_BASIC));
      addstr("\" je pro zpracovani prilis veliky. !!!");
      attrset(A_NORMAL);
      fclose(frw);
      COR;
      getch();
      return -1;
    } 
    
    if (flag_tmp) { /* ctu primo zdrojovy soubor */
      for (ind=0; ind!=new_l_file; ind++) { /* nacti si soubor do pomocneho bufferu */
        if (!(fread(&c, sizeof(c), 1, frw))) {  /* dojde li k chybe vypis hlaseni a uvolni pom buffer */ 
          move(LINES-1, 0);
          attrset(A_STANDOUT);
          addstr("!!! Chyba pri cteni souboru \"");
          addstr(cutname(n, CUT_NAME));
          addstr("\". !!!");
	  attrset(A_NORMAL);
	  free((void *) new_file);
          COR; 
	  getch();
	  fclose(frw);
	  return -1;
        }
        new_file[ind]=c; 
      }
      fclose(frw);
    }
    else { /* pouzivam temp soubor */
      for (ind=0; ind!=new_l_file; ind++) { /* nacti si soubor do pomocneho bufferu */
        if (!(fread(&c, sizeof(c), 1, tmp))) {  /* dojde li k chybe vypis hlaseni a uvolni pom buffer */ 
          move(LINES-1, 0);
          attrset(A_STANDOUT);
          addstr("!!! Chyba pri cteni souboru \"");
          addstr(cutname(n, CUT_NAME));
          addstr("\". !!!");
	  attrset(A_NORMAL);
	  free((void *) new_file);
          COR; 
	  getch();
	  fclose(tmp);
	  return -1;
        }
        new_file[ind]=c; 
      }
      fclose(tmp);
    }
     
    /* pokud vse probehlo v poradku */
    free((void *) file); /* vylij soucasny buffer */
    if ((file=(unsigned char *) malloc(new_l_file+1)) == NULL) { /* naalokuj si ho znovu */
      for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
      move(LINES-1, 0);
      attrset(A_STANDOUT);
      addstr("!!! Fatalni chyba. Nedostatek pameti. Aplikace bude ukoncena. !!! ");
      attrset(A_NORMAL);
      fclose(frw);
      COR;
      getch();
      finish(0);
    } 
    
    for (hlp=0; hlp!=new_l_file; hlp++) file[hlp]=new_file[hlp]; /* prekopiruj si novy soubor do file */
    free((void *) new_file); /* vylij pomocny buffer */    
    
    /* nastav nove hodnoty */
    l_file=new_l_file;
    source_code=new_source_code;
    flag_crlf=new_crlf;

  return 1;
}

void basic(void)
{ int i;
  char strpage[19]; /* pro vypsani aktualni zobrazene stranky */
 
  erase();
  move(0, 1);
  addstr("xcz2cz  ");
  addch(' '); addch('[');
  if (modified == 1) addch('M');
  else addch('-');
  addch(']');
  sprintf(strpage, " Stranka: %ld", page);
  addstr(strpage);
  addstr("  Soubor: ");
  addstr(cutname(name, CUT_NAME_BASIC));
  if (strcmp(name, "") != 0) {
    addstr(" (");
    addstr(ret_charcod(source_code));
    addstr(")");
  }
 
  for (i=0; i!=COLS; i++) { mvaddch(LINES-3, i, '-'); mvaddch(1, i, '-'); }
  move(LINES-2, 0);
  addch(' '); attrset(A_STANDOUT);
  addch('R'); attrset(A_NORMAL);
  addstr(" Nacti soubor      "); attrset(A_STANDOUT);
  addch('P'); attrset(A_NORMAL);
  addstr(" Preved a uloz      "); attrset(A_STANDOUT);
  addch('S'); attrset(A_NORMAL);
  addstr(" Uloz          "); attrset(A_STANDOUT);
  addch('Q'); attrset(A_NORMAL);
  addstr(" Konec"); 
  move(LINES-1, 0);
  addch(' '); attrset(A_STANDOUT);
  addch('K'); attrset(A_NORMAL);
  addstr(" Vstupni kodovani  "); attrset(A_STANDOUT);
  addch('D'); attrset(A_NORMAL);
  addstr(" Dopln hacky&carky  "); attrset(A_STANDOUT);
  addch('I'); attrset(A_NORMAL);
  addstr(" Napoveda  "); attrset(A_STANDOUT);
  addstr("sipky"); attrset(A_NORMAL);
  addstr(" Posun v textu"); 
  COR;
  refresh();
}

int ret_num(int z)
{ return z-48; }

int recod_save(void) 
{ int i;
  char vystup[255];
  FILE *out;
  struct stat s;
  char yesno; /* ano/ne */

  erase();
  move(0, 1);
  addstr("xcz2cz verze " VER "   soubor: ");
  addstr(cutname(name, CUT_NAME_BASIC));
  if (strcmp(name, "") != 0) {
    addstr(" (");
    addstr(ret_charcod(source_code));
    addstr(")");
  }
  for (i=0; i!=COLS; i++) { 
     mvaddch(LINES-3, i, '-'); 
     mvaddch(1, i, '-'); 
     mvaddch(LINES-2, i, ' '); } 
  
  move(5, 6);
  addstr("Zvolte kodovani, do ktereho bude soubor prelozen: ");
  move(6, 6);
  addstr("------------------------------------------------- ");
  move(8, 4);
  attrset(A_STANDOUT);
  addch('1');
  attrset(A_NORMAL);
  addstr(".....ISO-8859-2");
  move(9, 4);
  attrset(A_STANDOUT);
  addch('2');
  attrset(A_NORMAL);
  addstr(".....Win-1250");
  move(10, 4);
  attrset(A_STANDOUT);
  addch('3');
  attrset(A_NORMAL);
  addstr(".....PCLatin");
  move(11, 4);
  attrset(A_STANDOUT);
  addch('4');
  attrset(A_NORMAL);
  addstr(".....Kamenickych");
  move(12, 4);
  attrset(A_STANDOUT);
  addch('5');
  attrset(A_NORMAL);
  addstr(".....7-bit ASCII"); 
  move(13, 4);
  attrset(A_STANDOUT);
  addch('6');
  attrset(A_NORMAL);
  addstr(".....postfixove sprezky");
  move(14, 4);
  attrset(A_STANDOUT);
  addch('7');
  attrset(A_NORMAL);
  addstr(".....prefixove sprezky");
  move(15, 4);
  attrset(A_STANDOUT);
  addch('8');
  attrset(A_NORMAL);
  addstr(".....TeX-ovska konvence");
  move(LINES-1, 1);
  attrset(A_STANDOUT);
  addch('U');
  attrset(A_NORMAL);
  addstr(" Zpet na hlavni obrazovku");
  COR;

  destination_code=getch();
  while ((destination_code > '8') || (destination_code < '1')) { 
    if (tolower(destination_code) == 'u') return -1; 
    destination_code=getch();
  }
  destination_code=ret_num(destination_code);
  
  erase();
  move(0, 1);
  addstr("xcz2cz verze " VER "   soubor: ");
  addstr(cutname(name, CUT_NAME_BASIC));
  if (strcmp(name, "") != 0) {
    addstr(" (");
    addstr(ret_charcod(source_code));
    addstr(")");
  }
  for (i=0; i!=COLS; i++) { 
    mvaddch(LINES-3, i, '-'); 
    mvaddch(1, i, '-'); 
    mvaddch(LINES-2, i, ' ');
  }
 
  move(5, 6);
  addstr("Zvolte konce radku, ktere budou pouzity: ");
  move(6, 6);
  addstr("---------------------------------------- ");
  move(8, 4);
  attrset(A_STANDOUT);
  addch('0');
  attrset(A_NORMAL);
  addstr(" ... <LF>");
  move(9, 4);
  attrset(A_STANDOUT);
  addch('1');
  attrset(A_NORMAL);
  addstr(" ... <CR><LF>");
  move(12, 4);
  addstr("Puvodni soubor pouzival ");
  if (flag_crlf == 1) addstr("<CR><LF>.");
  else addstr("<LF>.");
  move(LINES-1, 1);
  attrset(A_STANDOUT);
  addch('U');
  attrset(A_NORMAL);
  addstr(" Zpet na hlavni obrazovku");
  COR;  
 
  l_ends=getch();
  while ((l_ends > '1') || (l_ends < '0')) { 
    if (tolower(l_ends) == 'u') return -1; 
    l_ends=getch();
  }
  l_ends=ret_num(l_ends);
  
  erase();
  move(0, 1);
  addstr("xcz2cz verze " VER "   soubor: ");
  addstr(cutname(name, CUT_NAME_BASIC));
  if (strcmp(name, "") != 0) {
    addstr(" (");
    addstr(ret_charcod(source_code));
    addstr(")");
  }
  for (i=0; i!=COLS; i++) { 
    mvaddch(LINES-3, i, '-'); 
    mvaddch(1, i, '-'); 
    mvaddch(LINES-1, i, ' ');
  }
 
  move(6, 6);
  addstr("-------------------------------- ");
  move(LINES-2, 1);
  addstr("(Pokud jmeno nezadate, pak se text ulozi do puvodniho souboru a prepise ho).");
  move(LINES-1, 1);
  attrset(A_STANDOUT);
  addstr("^U");
  attrset(A_NORMAL);
  addstr(" Zpet na hlavni obrazovku");
  move(5, 6);
  addstr("Zvolte jmeno vystupniho souboru: ");
  echo();
  if (my_getstr(vystup) == -1) return -1;
  noecho();
  
  if (stat(vystup, &s) == 0) { /* existuje-li soubor vystup */
    for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
    move(LINES-2, 1);
    addstr("Soubor \"");
    addstr(vystup);
    addstr("\" uz existuje. Chcete jej prepsat? (a/n)");
    COR;
    yesno=getch();
    if (strcmp(unctrl(yesno), "^U") == 0) return -1; 
    if ((yesno != 'A') && (yesno != 'a')) return -1;
  }
  
  for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
  for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' '); 
  
  if (strcmp(vystup, "") == 0) { /* soubor ma stejne jmeno jako puvodni */
    strcpy(vystup, name);
    if ((destination_code == source_code) && (l_ends == flag_crlf) && (modified == 0)) {
      /* pokud chci vysledny text v DEST a lf a text nebyl modifikovan neni potreba cokoliv delat */
      move(LINES-1, 2);
      attrset(A_STANDOUT);
      addstr("!!! Text je stejny jako text zdrojovy - neni potreba ukladat. !!!");
      attrset(A_NORMAL);
      COR;
      getch();
      return -1;
    }
  }
  
  if ((tmp=tmpfile()) == NULL) { 
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Nemohu otevrit temp soubor !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;
  } 
  
  for (ind=0; ind!=l_file; ind++) {
    if (!(fwrite(&file[ind], sizeof(file[ind]), 1, tmp))) {
       move(LINES-1, 0);
       attrset(A_STANDOUT);
       addstr("!!! Nemohu zapsat do temp souboru !!!");
       attrset(A_NORMAL);
       COR;
       getch();
       return -1;
    }
  }
  
  if ((fseek(tmp, 0, SEEK_SET)) != 0) { 
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Chyba pri presunu v temp souboru !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;
  }
    
  if ((out=fopen(vystup, "wb")) == NULL) {
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Nemohu otevrit soubor \"");
    addstr(cutname(vystup, CUT_NAME));
    addstr("\" pro zapis. !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;
  }   
  
  if (recode(tmp, out, DEST, destination_code, l_ends) != 0) { /* preved */
    move(LINES-2, 0);
    attrset(A_STANDOUT);
    addstr("!!! Chyba pri prevodu souboru \"");
    addstr(cutname(name, CUT_NAME));
    addstr("\".");
    move(LINES-1, 0);
    addstr("Kodovaci tabulka neni implementovana. !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;  
  }
  fclose(tmp);        
  fclose(out);
  return 0;
}

long show(long from_where) /* fce bere jako argument od ktereho znaku se ma vypisovat */
                           /* vraci index znaku, kterym se ma pokracovat na dalsi obrazovce */
{ int line_pos=0, j;
  long character;
  long lengh, strlengh;
  int y=2; /* souradnice radku */
  int poz; /* kde zacinam vypisovat */
  int word_begin; /* kvuli plneni pole - kde se nachazi prvek */
 
  if (file == NULL) return 0; 
  if (l_file == 0) {
    move(2, 0);
    attrset(A_STANDOUT);
    addstr("[prazdny soubor]");
    attrset(A_NORMAL);
    return 0;
  }
  
  character=from_where;
  move(y, 0);
  poz=character;
  line_pos=0;
  
  while ((y < LINES-3)) {
    if (isspace(file[character]) == 0) {
      word_begin=character; /* zacatek slova */
      strlengh=0;
      
      while (isspace(file[character]) == 0) { /* spocitej pocet pismen ve slove */
	 strlengh++; 
	 character++;
	 if (character == l_file) break; /* osetreni konce souboru */
      }
      
      if ( strlengh > COLS) { /* nevejde-li se slovo na radek */
        if ( strlengh > ((COLS)*(LINES-3-y))) { /* jestlize je retez tak dlouhy, ze presahne obrazovku */
	  lengh=((LINES-4-y)*(COLS)+((COLS)-line_pos)); /* kolik znaku zbyva do konce obrazovky */
	  for (j=0; j!=lengh; j++) { /* vypis vsecky pismena slova, ktere se vlezou na obrazovku */
	    getyx(stdscr, wherey[j+word_begin], wherex[j+word_begin]);
	    addch(file[j+word_begin]);
	  }
	  COR;
          return word_begin+lengh; /* skonci */
        } 
        y+=((strlengh+line_pos)/COLS); /* na kolik radku se slovo roztahne */
	line_pos=((strlengh+line_pos)%COLS);/* jaka bude pozice na poslednim radku */
	if (line_pos == 0) y++;
      }
      else {
        if ((line_pos+strlengh) > COLS) { /* vejde-li se slovo na radek, ale nevejde-li se na aktualni radek */
          y++;
	  move(y, 0); /* poskoc na dalsi radek */
	  line_pos=strlengh; /* a pozici na radku nastav podle delky slova */
        } 
	else { 
	  line_pos+=strlengh; /* vejde-li se slovo na aktualni radek */
        }
      }
      if (y < LINES-3) { /* vypis slovo */
        for (j=0; j!=strlengh; j++) {
	  getyx(stdscr, wherey[j+word_begin], wherex[j+word_begin]);
          addch(file[j+word_begin]);
	}
      }
      else { COR; return word_begin; }
    }
    else { /* reseni mezer a LF */
      while (isspace(file[character]) != 0) {
        if (y > LINES-4) { COR; return character; }
	if (file[character] == '\t') line_pos+=TABSIZE;
	else line_pos++;
	if ((line_pos >= COLS) || (file[character] == LF)) {
	  y++;
	  move(y, 0);
	  line_pos=0;
	}
	else { getyx(stdscr, wherey[character], wherex[character]); 
	       addch(file[character]);
	     } 
	character++;
	if (character == l_file) break;
      }	      
    }
    if (character == l_file) break;
  }	
  COR;
  return character;
}

int quick_save(void) 
{ 
  FILE *out;
  int i; 
  
  for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
  for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' ');
   
  /*jestlize je zdrojove kodovani rovno ASCII, ulozi se to jak to lezi a bezi (v UNIXu ISO2, v DOSu PCLatin) */
  if (((source_code == DEST) || (source_code == 5)) && (flag_crlf == 0)) { /* pokud je zdrojovy soubor DEST a lf, nebo ascii a lf */
    if ((out=fopen(name, "wb")) == NULL) {
        move(LINES-1, 0);
	attrset(A_STANDOUT);
        addstr("!!! Nemohu otevrit soubor \"");
        addstr(cutname(name, CUT_NAME));
        addstr("\" pro zapis. !!!");
	attrset(A_NORMAL);
        COR;
        getch();
        return -1;
     }
      
     for (ind=0; ind!=l_file; ind++) {
       if (!(fwrite(&file[ind], sizeof(file[ind]), 1, out))) {
         move(LINES-1, 0);
	 attrset(A_STANDOUT);
         addstr("!!! Nemohu zapsat do souboru \"!!!");
	 addstr(cutname(name, CUT_NAME));
	 addstr("\" !!!");
         attrset(A_NORMAL);
         COR;
         getch();
         return -1;
       }       
     }
     fclose(out);
     return 0;
  }
            
  /* jestlize neni text ascii nebo dest */
  if ((tmp=tmpfile()) == NULL) { /* otevri temp */
    for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
     move(LINES-1, 0);
     attrset(A_STANDOUT);
     addstr("!!! Nemohu otevrit soubor temp pro zapis. !!!");
     attrset(A_NORMAL);
     COR;
     getch();
     return -1;
  } 
  
  for (ind=0; ind!=l_file; ind++) { /* vylij do nej buffer */
    if (!(fwrite(&file[ind], sizeof(file[ind]), 1, tmp))) {
      move(LINES-1, 0);
      attrset(A_STANDOUT);
      addstr("!!! Nemohu zapsat do temp souboru !!!");
      attrset(A_NORMAL);
      COR;
      getch();
      return -1;
    }       
  }
   
   if ((fseek(tmp, 0, SEEK_SET)) != 0) { /* prejdi znovu na zacatek */ 
      for (i=0; i!=COLS; i++) mvaddch(LINES-3, i, ' ');
      move(LINES-1, 0);
      attrset(A_STANDOUT);
      addstr("!!! Chyba pri presunu v temp souboru !!!");
      attrset(A_NORMAL);
      COR;
      getch();
      return -1;
    }
  
  if ((out=fopen(name, "wb")) == NULL) { /* otevri vystupni soubor */
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Nemohu otevrit soubor \"");
    addstr(cutname(name, CUT_NAME));
    addstr("\" pro zapis. !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;
  }   

  if (recode(tmp, out, DEST, source_code, flag_crlf) != 0) { /* preved tmp do vystupniho */
    move(LINES-2, 0);
    attrset(A_STANDOUT);
    addstr("! Chyba pri prevodu souboru \"");
    addstr(cutname(name, CUT_NAME));
    addstr("\".");
    move(LINES-1, 0);
    addstr("Kodovaci tabulka neni implementovana. !");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;  
  }
  fclose(tmp);        
  fclose(out);
  
  return 0;
}

int change_code(void)
{ int i; 
  int new_source_code;
  unsigned char c;
  
  erase();
  move(0, 1);
  addstr("xcz2cz verze " VER "   soubor: ");
  addstr(cutname(name, CUT_NAME_BASIC));
  if (strcmp(name, "") != 0) {
    addstr(" (");
    addstr(ret_charcod(source_code));
    addstr(")");
  }
  for (i=0; i!=COLS; i++) { 
     mvaddch(LINES-3, i, '-'); 
     mvaddch(1, i, '-'); 
     mvaddch(LINES-2, i, ' '); 
  } 
  
  move(5, 6);
  addstr("Zvolte kodovani, ve kterem je zdrojovy soubor: ");
  move(6, 6);
  addstr("--------------------------------------------- ");
  move(8, 4);
  attrset(A_STANDOUT);
  addch('1');
  attrset(A_NORMAL);
  addstr(".....ISO-8859-2");
  move(9, 4);
  attrset(A_STANDOUT);
  addch('2');
  attrset(A_NORMAL);
  addstr(".....Win-1250");
  move(10, 4);
  attrset(A_STANDOUT);
  addch('3');
  attrset(A_NORMAL);
  addstr(".....PCLatin");
  move(11, 4);
  attrset(A_STANDOUT);
  addch('4');
  attrset(A_NORMAL);
  addstr(".....Kamenickych");
  move(LINES-1, 1);
  attrset(A_STANDOUT);
  addch('U');
  attrset(A_NORMAL);
  addstr(" Zpet na hlavni obrazovku");
  COR; 
 
  new_source_code=getch();
  while ((new_source_code > '4') || (new_source_code < '1')) { 
    if (tolower(new_source_code) == 'u') return -1; 
    new_source_code=getch();
  }
  new_source_code=ret_num(new_source_code);  
 
  for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
  for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' ');
 
  if (new_source_code == source_code) {
    move(LINES-1, 1);
    attrset(A_STANDOUT);
    addstr("!!! Soubor ma kodovani ");
    addstr(ret_charcod(new_source_code));
    addstr(" jiz nastaveno. !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;
  }
  
  if ((frw=fopen(name, "rb")) == NULL) {
      move(LINES-1, 1);
      attrset(A_STANDOUT);
      addstr("!!! Nemohu otevrit soubor \"");
      addstr(cutname(name, CUT_NAME));
      addstr("\". !!!");
      attrset(A_NORMAL);
      COR;
      getch();
      return -1;
    }
  
  if ((tmp=tmpfile()) == NULL) {
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Nemohu otevrit temp soubor !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;
  } 
  
  if (recode(frw, tmp, new_source_code, DEST, 0) != 0) { /* preved na DEST a lf */
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Chyba pri prevodu souboru \"");
    addstr(cutname(name, CUT_NAME));
    addstr("\". !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;  
  }

  fclose(frw);
  
  if ((fseek(tmp, 0, SEEK_SET)) != 0) { /* prejdi znovu na zacatek */ 
    move(LINES-1, 0);
    attrset(A_STANDOUT);
    addstr("!!! Chyba pri presunu v temp souboru !!!");
    attrset(A_NORMAL);
    COR;
    getch();
    return -1;
  }
  
  for (ind=0; ind!=l_file; ind++) { /* nacti si soubor do bufferu */
    if (!(fread(&c, sizeof(c), 1, tmp))) {  /* dojde li k chybe vypis hlaseni a shod program */ 
      move(LINES-1, 0);
      attrset(A_STANDOUT);
      addstr("!!! Fatalni chyba pri cteni temp souboru !!!");
      attrset(A_NORMAL);
      COR;
      getch();
      finish(0);
    }
    file[ind]=c; 
  }
   
  fclose(tmp);
  source_code=new_source_code;

  return 0;
}
  
void info(void)
{ int g;

  erase();
  for (g=0; g!=COLS; g++) {
    mvaddch(0, g, '=');
    mvaddch(2, g, '=');
  }
  move(1, 4);
  addstr("xcz2cz  verze: " VER "                                Vlastimil Krejcir 2004");                          
  move(4, 4);
  addstr("Aplikace slouzi k prevadeni textovych souboru mezi ruznymi kodovanimi"); 
  move(5, 1); 
  addstr("ceskeho jazyka a doplnovani hacku a carek do textu. Aplikace je interaktivni");
  move(6, 1);
  addstr("verzi programu cz2cz - vyuziva jeho knihoven vcetne automaticke detekce kodo-");
  move(7, 1);
  addstr("vani a prevodnich tabulek.");
  
  move(9, 4);
  addstr("Popis ovladani: ");
  move(10, 4);
  addstr("--------------- ");
  move(11, 1);
  addstr("R: nacte novy soubor");
  move(12, 1);
  addstr("S: ulozi text do stavajiciho souboru v jeho puvodnim kodovani");
  move(13, 1);
  addstr("P: ulozi text do souboru zadaneho jmena a v zadanem kodovani");
  move(14, 1);
  addstr("K: manualni zmena kodovani textu zdrojoveho souboru;");
  move(15, 1);
  addstr("   pokud nevidite spravnou interpunkci, muzete nastavit kodovani zdrojoveho");
  move(16, 1);
  addstr("   souboru manualne - neni-li vysledek lepsi, pak kodovani, ve kterem je");
  move(17, 1);
  addstr("   zdrojovy soubor, neni pravdepodobne podporovano");
  move(18, 1);
  addstr("D: prejde do rezimu doplnovani hacku a carek");
  move(19, 1);
  addstr("Q: ukonci program");
  move(20, 1);
  addstr("Sipka nahoru, Sipka dolu: prohlizeni textu");
  COR;
  getch();
  
  erase();
  for (g=0; g!=COLS; g++) {
    mvaddch(0, g, '=');
    mvaddch(2, g, '=');
  }
  move(1, 4);
  addstr("xcz2cz  verze: " VER "                                Vlastimil Krejcir 2004");                          
  move(4, 4);
  addstr("Rezim doplnovani interpunkce:");
  move(5, 4);
  addstr("-----------------------------");
  move(6, 1);
  addstr("S: ulozi text do stavajiciho soubor v jeho puvodnim kodovani");
  move(7, 1);
  addstr("Q, R, T: doplni nad pismeno na pozici kurzoru carku");
  move(8, 1);
  addstr("W, F, G: doplni nad pismeno na pozici kurzoru hacek (v pripade 'u' krouzek)");
  move(9, 1);
  addstr("E, Space: u drive zmenenych pismen vrati puvodni znak (buffer 50 znaku zpet)");
  move(10, 1);
  addstr("Sipka vlevo: posun na predchozi znak (ktery je mozne zmenit)");
  move(11, 1);
  addstr("Sipka vpravo: posun na nasledujici znak (ktery je mozne zmenit)");
  move(12, 1);
  addstr("Sipka nahoru: skok na predchozi vhodny znak, minimalne vsak o radek vyse");
  move(13, 1);
  addstr("Sipka dolu: skok na dalsi vhodny znak, minimalne vsak o radek nize");
  move(14, 1);
  addstr("U: ukonci rezim vkladani interpunkce");
  move(17, 4);
  addstr("Dalsi informace:");
  move(18, 4); 
  addstr("----------------");
  move(19, 1);
  addstr("Kontakt na autora: xkrejci4@fi.muni.cz, krejvl@email.cz");
  move(20, 1);
  addstr("URL: http://www.fi.muni.cz/~xkrejci4/cz2cz/");
  move(21, 1);
  addstr("     http://www.fi.muni.cz/~xkrejci4/prog/");
  COR;
  getch();
}      

void status()
{ int i;

  for (i=0; i!=COLS; i++) { mvaddch(LINES-1, i, ' '); mvaddch(LINES-2, i, ' '); }
  move(LINES-2, 1);
  attrset(A_STANDOUT);
  addch('S');
  attrset(A_NORMAL);
  addstr(" Uloz         ");
  attrset(A_STANDOUT);
  addstr("Q, R, T");
  attrset(A_NORMAL);
  addstr(" carka            ");
  attrset(A_STANDOUT);
  addstr("E, Space");
  attrset(A_NORMAL);
  addstr(" puvodni znak");
  move(LINES-1, 1);
  attrset(A_STANDOUT);
  addch('U');
  attrset(A_NORMAL);
  addstr(" Konec dopl.  ");
  attrset(A_STANDOUT);
  addstr("W, F, G");
  attrset(A_NORMAL);
  addstr(" hacek (krouzek)  ");
  attrset(A_STANDOUT);
  addstr("");
  attrset(A_NORMAL);
  addstr("");    
}


int prev_char(void);

int next_char(void)
{ int set[26]= { 'e', 'r', 't', 'y', 'u', 'i', 'o', 'a', 's', 'd', 'z', 'c', 'n', 
                 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'A', 'S', 'D', 'Z', 'C', 'N'};
  int i, flag=0;
  
  do {
    flag=0;
    if (relpos >= (l_file-1)) { 
      if (cycle_flag == 1) { cycle_flag=0; return 0; }
      if (relpos > (l_file-1)) relpos--;
      cycle_flag=1; prev_char(); return 0;
    }
    if (relpos > next) {
      if (p_act->nxt == NULL) { /* pokud jeste dalsi obrazovka neni, tak priprav dalsi prvek seznamu */
	if ((p_act->nxt=(LIST *) malloc(sizeof(LIST))) == NULL) { /* priprava zasobniku */
          fprintf(stderr, "xcz2cz: Nedostatek pameti !!!\n");
          finish(0);
        }
        p_hlp=p_act;
        p_act=p_act->nxt;
        p_act->prv=p_hlp;
	p_act->nxt=NULL;
      }
      else p_act=p_act->nxt;

      p_act->val=position;
      page++;
      basic();
      status();
      position=next;
      next=show(next);
    }
    u_act=u_last;
    while (u_act != u_first) { /* projdi undo pozpatku */
      if (relpos == u_act->pstn) { flag=1; cycle_flag=0; break; }
      u_act=u_act->prv;
    }
    for (i=0; i!=26; i++) if (file[relpos] == set[i]) { flag=1; cycle_flag=0; break; }
    if (flag != 1) relpos++;
  } while (flag == 0);

  return 0; 
}  

int prev_char(void)
{ int set[26]= { 'e', 'r', 't', 'y', 'u', 'i', 'o', 'a', 's', 'd', 'z', 'c', 'n',
                 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'A', 'S', 'D', 'Z', 'C', 'N'}; 
  int i, flag=0;
 
  do {
    flag=0;
    if (relpos <= 0) {
      if (cycle_flag == 1) { cycle_flag=0; return 0; }
      if (relpos < 0) relpos++;
      cycle_flag=1;
      next_char(); return 0;
    }
    if (relpos < position) {
      page--;
      basic();
      status();
      position=p_act->val;
      next=show(p_act->val);
      p_act=p_act->prv;
    }
    u_act=u_last;
    while (u_act != u_first) { /* projdi undo pozpatku */
      if (relpos == u_act->pstn) { flag=1; cycle_flag=0; break; }
      u_act=u_act->prv;
    }
    for (i=0; i!=26; i++) if (file[relpos] == set[i]) { flag=1; cycle_flag=0; break; }
    if (flag != 1) relpos--;
  } while (flag == 0);
  return 0;
}  

int next_line(void)
{ int y_now;
  long counter;

  y_now=wherey[relpos];
  while (wherey[relpos] == y_now) {
    counter=relpos; 
    relpos++; next_char();
    if (counter == relpos) break;
  } 
  return 0;
}

int prev_line(void)
{ int y_now;
  long counter;

  y_now=wherey[relpos];
  while (wherey[relpos] == y_now) {
    counter=relpos;
    relpos--; prev_char(); 
    if (counter == relpos) break;
  }
  return 0;
}

void carka(void)
{ int change=1;
  char was;
   
  was=file[relpos];

#if UNIX
  switch (file[relpos]) {
    case 'e': file[relpos]=233; addch(233); break;
    case 'E': file[relpos]=201; addch(201); break;
    case 'y': file[relpos]=253; addch(253); break;
    case 'Y': file[relpos]=221; addch(221); break;
    case 'u': file[relpos]=250; addch(250); break;
    case 'U': file[relpos]=218; addch(218); break;
    case 'i': file[relpos]=237; addch(237); break;
    case 'I': file[relpos]=205; addch(205); break;
    case 'o': file[relpos]=243; addch(243); break;
    case 'O': file[relpos]=211; addch(211); break;
    case 'a': file[relpos]=225; addch(225); break;
    case 'A': file[relpos]=193; addch(193); break;
    default: change=0;
  }
#endif
#if DOS
   switch (file[relpos]) {
    case 'e': file[relpos]=130; addch(130); break;
    case 'E': file[relpos]=144; addch(144); break;
    case 'y': file[relpos]=236; addch(236); break;
    case 'Y': file[relpos]=237; addch(237); break;
    case 'u': file[relpos]=163; addch(163); break;
    case 'U': file[relpos]=233; addch(233); break;
    case 'i': file[relpos]=161; addch(161); break;
    case 'I': file[relpos]=214; addch(214); break;
    case 'o': file[relpos]=162; addch(162); break;
    case 'O': file[relpos]=224; addch(224); break;
    case 'a': file[relpos]=160; addch(160); break;
    case 'A': file[relpos]=181; addch(181); break;
    default: change=0;
  }
#endif
  
  if (change == 1) { 
    modified=1;
    if ((u_act=(UNDO *) malloc(sizeof(UNDO))) == NULL) { /* priprava zasobniku */
     fprintf(stderr, "xcz2cz: Nedostatek pameti !!!\n");
     finish(0);
    }
    u_last->nxt=u_act;
    u_act->pstn=relpos;
    u_act->ch=was;
    u_act->nxt=NULL;
    u_act->prv=u_last;
    u_last=u_act;

    relpos++; next_char(); status();
    move(0, 11); addch('M');
    move(wherey[relpos], wherex[relpos]);
  }  
}

void hacek(void)
{ int change=1;
  char was;
   
  was=file[relpos];

#if DOS
  switch (file[relpos]) {
    case 'e': file[relpos]=216; addch(216); break;
    case 'E': file[relpos]=183; addch(183); break;
    case 'r': file[relpos]=253; addch(253); break;
    case 'R': file[relpos]=252; addch(252); break;
    case 't': file[relpos]=156; addch(156); break;
    case 'T': file[relpos]=155; addch(155); break;
    case 'u': file[relpos]=133; addch(133); break;
    case 'U': file[relpos]=222; addch(222); break;
    case 's': file[relpos]=231; addch(231); break;
    case 'S': file[relpos]=230; addch(230); break;
    case 'd': file[relpos]=212; addch(212); break;
    case 'D': file[relpos]=210; addch(210); break;
    case 'z': file[relpos]=167; addch(167); break;
    case 'Z': file[relpos]=166; addch(166); break;
    case 'c': file[relpos]=159; addch(159); break;
    case 'C': file[relpos]=172; addch(172); break;
    case 'n': file[relpos]=229; addch(229); break;
    case 'N': file[relpos]=213; addch(213); break;
    default: change=0;
  }
#endif
#if UNIX
   switch (file[relpos]) {
    case 'e': file[relpos]=236; addch(236); break;
    case 'E': file[relpos]=204; addch(204); break;
    case 'r': file[relpos]=248; addch(248); break;
    case 'R': file[relpos]=216; addch(216); break;
    case 't': file[relpos]=187; addch(187); break;
    case 'T': file[relpos]=171; addch(171); break;
    case 'u': file[relpos]=249; addch(249); break;
    case 'U': file[relpos]=217; addch(217); break;
    case 's': file[relpos]=185; addch(185); break;
    case 'S': file[relpos]=169; addch(169); break;
    case 'd': file[relpos]=239; addch(239); break;
    case 'D': file[relpos]=207; addch(207); break;
    case 'z': file[relpos]=190; addch(190); break;
    case 'Z': file[relpos]=174; addch(174); break;
    case 'c': file[relpos]=232; addch(232); break;
    case 'C': file[relpos]=200; addch(200); break;
    case 'n': file[relpos]=242; addch(242); break;
    case 'N': file[relpos]=210; addch(210); break;
    default: change=0;
  }
#endif
  if (change == 1) { 
    modified=1;
    if ((u_act=(UNDO *) malloc(sizeof(UNDO))) == NULL) { /* priprava zasobniku */
     fprintf(stderr, "xcz2cz: Nedostatek pameti !!!\n");
     finish(0);
    }
    u_last->nxt=u_act;
    u_act->pstn=relpos;
    u_act->ch=was;
    u_act->nxt=NULL;
    u_act->prv=u_last;
    u_last=u_act;

    relpos++; next_char(); status();
    move(0, 11); addch('M');
    move(wherey[relpos], wherex[relpos]);
  }  
}

void take_undo(void)
{ 
  u_act=u_last;    
  while (u_act != u_first) { 
    if (relpos == u_act->pstn) {
      file[relpos]=u_act->ch;
      addch(u_act->ch);
      modified=1;
      /* uvolni prvek ze seznamu */
      if (u_act == u_last) { 
        u_last=u_act->prv;
	u_last->nxt=NULL;
      }
      else {
        u_hlp=u_act->prv;
        u_hlp->nxt=u_act->nxt;
        u_hlp=u_act->nxt;
        u_hlp->prv=u_act->prv;
      }
      free((void *) u_act);
      
      relpos++; next_char(); status();
      move(0, 11); addch('M');
      move(wherey[relpos], wherex[relpos]);
      u_act=u_first;
    }
    else u_act=u_act->prv;
  }
}  

int add_interpunct(void)
{ int c;

  /* q, r, t - carka */
  /* w, f, g - hacek, krouzek */
  /* e, space - zpet */
  status();
  relpos=position;
  next_char();
  move(wherey[relpos], wherex[relpos]);
  while ((c=getch())) {
    if (isalpha(c) != 0) c=tolower(c);
    switch (c) {
      case KEY_RIGHT: relpos++;
                      next_char();
                      status();
		      move(wherey[relpos], wherex[relpos]);         
		      break;
      case KEY_LEFT: relpos--;
                     prev_char();
                     status();
		     move(wherey[relpos], wherex[relpos]);
		     break;
      case KEY_UP: if (cycle_flag == 0) prev_line();
                   status();
		   move(wherey[relpos], wherex[relpos]);
		   break;
      case KEY_DOWN: if (cycle_flag == 0) next_line();
                     status();
		     move(wherey[relpos], wherex[relpos]);
		     break;
      case 'q': carka();
                break;		     
      case 'r': carka();
                break;
      case 't': carka();
                break;
      case 'w': hacek();
                break;		     
      case 'f': hacek();
                break;
      case 'g': hacek();
                break;
      case 'e': take_undo();
                break;
      case ' ': take_undo();
                break;
      case 's': if (modified == 1) {
		  if (quick_save() == 0) modified=0;
		}
		basic();
		next=show(position);
		status();
		move(wherey[relpos], wherex[relpos]);   
	        break;
      case 'u': return 0;
    }
  }      
  return 0; 
}

int main(int argc, char **argv)
{ 
  char oldname[MAX_FILE_PATH];
  unsigned char c;  
  int hlp, i; /* pomocna promenna */
  int set[26]= { 'e', 'r', 't', 'y', 'u', 'i', 'o', 'a', 's', 'd', 'z', 'c', 'n',
                 'E', 'R', 'T', 'Y', 'U', 'I', 'O', 'A', 'S', 'D', 'Z', 'C', 'N'};
  
  file=NULL;
 
  if (argc > 2) { printf("xcz2cz: Program spoustejte vzdy bez parametru nebo s jednim parametrem <soubor>\n");
                  return -1;
		}
			
  if (argc == 2) {
    if ((frw=fopen(argv[1], "rb")) == NULL) {
      printf("xcz2cz: Nemohu otevrit soubor %s.\n", argv[1]);
      return -1;
    }
    hlp=0;
    
    l_file=0;
    while (fread(&c, sizeof(c), 1, frw)) {
      if ((flag_crlf == 0) && (hlp == CR) && (c == LF)) flag_crlf=1;
      hlp=c;
      if (c != CR) l_file++;
    }
    
    if ((file=(unsigned char *) malloc(l_file+1)) == NULL) {
      printf("xcz2cz: Soubor %s je pro editaci prilis veliky.\n"
             "Pouzijte prosim neinteraktivniho zpracovani pomoci programu cz2cz.\n", argv[1]);
      fclose(frw);
      return -1;
    }
    
    if ((fseek(frw, 0, SEEK_SET)) != 0) { 
      fprintf(stderr, "xcz2cz: Chyba pri presunu v souboru %s.\n", argv[1]);
      return -1;
    }
    
    hlp=autod(argv[1]);
    if (hlp > 0) source_code=hlp;
    else source_code=1;
    
    if ((source_code != DEST) && (source_code <= 5)) {
      if ((tmp=tmpfile()) == NULL) {
        printf("xcz2cz: Nemohu otevrit temp soubor pro zapis.\n");
        return -1;
      } 
      
      if ((hlp=recode(frw, tmp, source_code, 1, 0)) != 0) { /* preved na iso a lf */
        fprintf(stderr, "xcz2cz: Chyba pri prevodu souboru %s.\n", argv[1]);
	return -1;  
      }
    
      flag_tmp=0;
    }
    fclose(frw);
    
    if (flag_tmp == 1) { /* soubor temp nebyl pouzit */
      if ((frw=fopen(argv[1], "rb")) == NULL) {
        printf("xcz2cz: Nemohu otevrit soubor %s.\n", argv[1]);
        return -1;
      }
      for(ind=0; ind!=l_file; ind++) {
        if (!(fread(&c, sizeof(c), 1, frw))) {  
          fprintf(stderr, "xcz2cz: Chyba pri cteni souboru %s.\n", argv[1]);
	  return -1;
        }
        file[ind]=c; 
      }
      fclose(frw);
    }
    else {
      if ((fseek(tmp, 0, SEEK_SET)) != 0) {
        printf("xcz2cz: Nemohu se presunout v temp soubor.\n");
        return -1;
      }
      for(ind=0; ind!=l_file; ind++) {
        if (!(fread(&c, sizeof(c), 1, tmp))) {  
          fprintf(stderr, "xcz2cz: Chyba pri cteni souboru %s.\n", argv[1]);
	  return -1;
        }
        file[ind]=c; 
      }
      fclose(tmp);
    }
  }
    
  (void) signal(SIGINT, finish);
  
  initscr();
  nonl();
  cbreak();
  noecho();
  keypad(stdscr, TRUE);
  
  if ((p_act=(LIST *) malloc(sizeof(LIST))) == NULL) { /* priprava zasobniku */
    fprintf(stderr, "xcz2cz: Nedostatek pameti !!!\n");
    finish(0);
  }
  p_first=p_act;
  p_act->prv=NULL;
  p_act->nxt=NULL;
  p_act->prv=NULL;
  p_act->val=0; /* prvni obrazovak zacina od nulteho znaku */
  
  
  if ((u_act=(UNDO *) malloc(sizeof(UNDO))) == NULL) { /* priprava zasobniku */
    fprintf(stderr, "xcz2cz: Nedostatek pameti !!!\n");
    finish(0);
  }
  u_first=u_act; /* u_first je inicialni = pusobi jako zarazka a pro mazani */
  u_last=u_act;
  u_act->nxt=NULL;
  u_act->prv=NULL;  

  page=0; /* cislovani stranek zacina od nuly */
  if (argc == 2) { 
    strcpy(name, argv[1]);  
    basic();
    if ((wherex=(int *) malloc(l_file * sizeof(int) + 1)) == NULL) finish(-1); /* alokuj buffery pro pozice znaku */
    if ((wherey=(int *) malloc(l_file * sizeof(int) + 1)) == NULL) finish(-1);
    position=0;
    next=show(0);  /* vypis prvni obrazovku */
  }
  else { basic(); strcpy(name, ""); }
  
  while ((hlp=getch())) {
    if (isalpha(hlp) != 0) hlp=tolower(hlp);
    switch (hlp) {
      case KEY_UP: if (p_act->prv != NULL) { /* pokud nejsem na zacatku seznamu */
                     page--; /* jen counter stranek */
                     basic();
		     p_act=p_act->prv;
		     position=p_act->val;
		     next=show(p_act->val);
		   }
                   break;
      case KEY_DOWN: if (next < l_file) {
                  if (p_act->nxt == NULL) { /* pokud jeste dalsi obrazovka neni, tak priprav dalsi prvek seznamu */
	            if ((p_act->nxt=(LIST *) malloc(sizeof(LIST))) == NULL) { /* priprava zasobniku */
                      fprintf(stderr, "xcz2cz: Nedostatek pameti !!!\n");
                      finish(0);
                    }
		    p_hlp=p_act;
		    p_act=p_act->nxt;
		    p_act->prv=p_hlp;
		    p_act->nxt=NULL;
		  }
		  else p_act=p_act->nxt;
		  page++;
		  basic();
		  position=next;
		  next=show(next);
		  p_act->val=position; 
		}
		break;
      case 'r': for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
                for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' ');
		move(LINES-1, 1);
		attrset(A_STANDOUT);
		addstr("^U");
		attrset(A_NORMAL);
		addstr(" Zpet");
		if ((modified == 1) && (file != NULL)) {
		  move(LINES-2, 1);
                  addstr("Chcete nejdrive ulozit otevreny soubor? (a/n)");
                  COR;
                  hlp=getch();
                  if ((hlp != 'N') || (hlp != 'n')) { 
		    if (quick_save() == -1) { basic(); next=show(position); break; } 
		    move(LINES-1, 1);
		    attrset(A_STANDOUT);
		    addstr("^U");
		    attrset(A_NORMAL);
		    addstr(" Zpet");
		  }
		  if (strcmp(unctrl(hlp), "^U") == 0) { basic(); next=show(position); break; }
                }  
		for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
                move(LINES-2, 1);
		addstr("Jmeno souboru: ");
		echo();
		strcpy(oldname, name);
                if (my_getstr(name) == -1) { 
		  noecho();
		  strcpy(name, oldname);
		  basic();
		  next=show(position);
		  break;
		}
		noecho();
		for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' ');    
		if (fil_open(name) == -1) { 
		  strcpy(name, oldname); 
		  basic(); 
		  next=show(position);  
		}
		else { /* soubor se podarilo vporadku otevrit */
		  modified=0; 
		  basic();
		  p_act=p_first;
		  while (p_act->nxt != NULL) { /* uvolni ukazatel obrazovek */
		    p_act=p_act->nxt;
		    free((void *) p_first);
		    p_first=p_act;
		  }
		  free((void *) p_act);
		  u_act=u_first;
		  while (u_act->nxt != NULL) { /* vyprazdni seznam undo */
		    u_act=u_act->nxt;
		    free((void *) u_first);
		    u_first=u_act;
		  }
		  free((void *) u_act);
		  free((void *) wherex);
		  free((void *) wherey);
		  if ((p_act=(LIST *) malloc(sizeof(LIST))) == NULL) { /* priprava zasobniku */
                    fprintf(stderr, "xcz2cz: Nedostatek pameti !!!\n");
                    finish(-1);
                  }
                  p_first=p_act;
                  p_act->prv=NULL;
                  p_act->nxt=NULL;
                  p_act->val=0; /* prvni obrazovak zacina od nulteho znaku */
		  if ((u_act=(UNDO *) malloc(sizeof(UNDO))) == NULL) { /* priprava zasobniku */
                    fprintf(stderr, "xcz2cz: Nedostatek pameti !!!\n");
                    finish(0);
                  }
                  u_first=u_act; /* u_first je inicialni = pusobi jako zarazka a pro mazani */
                  u_last=u_act;
                  u_act->nxt=NULL;
                  u_act->prv=NULL;  
		  if ((wherex=(int *) malloc(l_file * sizeof(int) +1)) == NULL) finish(-1);
                  if ((wherey=(int *) malloc(l_file * sizeof(int) +1)) == NULL) finish(-1); 
		  next=show(0); 
		  position=0;  
		}
		break;
      case 'p': for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
                for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' ');
                if (strcmp(name, "") == 0) {
                   move(LINES-1, 1);
		   attrset(A_STANDOUT);
		   addstr("!!! Neni nacten zadny soubor. !!!");
		   attrset(A_NORMAL);
		   getch();
		}
		else {
                  if (recod_save() == 0) modified=0;
		}
		basic();
		next=show(position);
		break;
      case 's': for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
                for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' ');
                if (strcmp(name, "") == 0) {
                  move(LINES-1, 1);
                  attrset(A_STANDOUT);
		  addstr("!!! Neni nacten zadny soubor. !!!");
		  attrset(A_NORMAL);
		  getch(); 
		}
		else {
		  if (modified == 1) {
		    if (quick_save() == 0) modified=0;
		  }
		}
		basic();
		next=show(position);
		break;
      case 'k': for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
                for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' '); 
                if (strcmp(name, "") == 0) {
                  move(LINES-1, 1);
		  attrset(A_STANDOUT);
		  addstr("!!! Neni nacten zadny soubor. !!!");
		  attrset(A_NORMAL);
		  getch();
		}
		else {
                  change_code();  
		}
		basic();
		next=show(position);
		break;	
      case 'i': info();
                basic();
                next=show(position);
		break;
      case 'd': for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
                for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' '); 
                hlp=0;
		for (ind=0; ind!=l_file; ind++) {
		  for (i=0; i!=26; i++) if (file[ind] == set[i]) { hlp=1; break; }
		  if (hlp == 1) break;
		}
		if (hlp == 0) { /* neobsahuje-li soubor zadny znak, ktery by se dal doplnit */
		  move(LINES-1, 1);
		  attrset(A_STANDOUT);
		  addstr("!!! Soubor neobsahuje zmenitelne znaky. !!!");
		  attrset(A_NORMAL);
		  getch();
		  basic();
		  next=show(position);
		  break;
		} 
		if (strcmp(name, "") == 0) {
                  move(LINES-1, 1);
		  attrset(A_STANDOUT);
		  addstr("!!! Neni nacten zadny soubor. !!!");
		  attrset(A_NORMAL);
		  getch();
		}
		else {
                  add_interpunct(); 
		}
		basic();
		next=show(position);
		break;	
      case 'q': for (i=0; i!=COLS; i++) mvaddch(LINES-2, i, ' ');
                for (i=0; i!=COLS; i++) mvaddch(LINES-1, i, ' '); 
                move(LINES-1, 1);
		attrset(A_STANDOUT);
		addstr("^U");
		attrset(A_NORMAL);
		addstr(" Zpet");
		if (strcmp(name, "") != 0) {
		  if (modified == 1) {
                    move(LINES-2, 1);
		    addstr("Soubor byl modifikovan a nebyl ulozen. Prejete si zmeny ulozit? (a/n) ");
		    COR;
		    hlp=getch();
		    if (strcmp(unctrl(hlp), "^U") == 0) { basic(); next=show(position); break;} 
		    if ((hlp != 'n') && (hlp != 'N')) { 
		      if (quick_save() == -1) { basic(); next=show(position); break; } 
		    }
		  } 
                  free((void *) file);
		  free((void *) wherex);
		  free((void *) wherey);
		  p_act=p_first;
		  while (p_act->nxt != NULL) { /* uvolni ukazatel obrazovek */
		    p_act=p_act->nxt;
		    free((void *) p_first);
		    p_first=p_act;
		  }
		  free((void *) p_act);
		  
		}
                finish(0);		
    }
  }		
  
  finish(0); 
  return 0;
}




